package cn.fw.shirasawa.rpc.member;

import cn.fw.common.util.MobileUtil;
import cn.fw.data.base.domain.common.Message;
import cn.fw.member.sdk.api.AccountApi;
import cn.fw.member.sdk.api.FunctionApi;
import cn.fw.member.sdk.api.MemberApi;
import cn.fw.member.sdk.vo.AccountInfoVo;
import cn.fw.member.sdk.vo.MemberInfoVo;
import cn.fw.member.sdk.vo.MemberPhoneInfoVo;
import cn.fw.member.sdk.vo.MobileLocation;
import cn.fw.shirasawa.rpc.AbsBaseRpcService;
import cn.fw.shirasawa.rpc.member.dto.MemberUserDTO;
import cn.fw.shirasawa.rpc.member.dto.UserAccountDTO;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.concurrent.CompletableFuture;

import static org.apache.commons.lang3.Validate.isTrue;

/**
 * 会员服务
 *
 * @author kurisu
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MemberRpcService extends AbsBaseRpcService {
    /**
     * 会员服务
     */
    private final MemberApi memberApi;
    private final AccountApi accountApi;
    private final FunctionApi functionApi;

    @Value("${spring.cache.custom.global-prefix}:rpc:member")
    private String keyPrefix;

    /**
     * 根据会员ID获取会员信息
     *
     * @param userId 会员ID
     * @return 会员信息
     */
    public MemberUserDTO user(final Long userId) {
        if (userId == null || userId <= 0) {
            return null;
        }
        try {
            final Message<MemberPhoneInfoVo> msg = memberApi.getMemberPhoneById(userId);
            if (!msg.isSuccess()) {
                log.warn("调用Member系统[根据会员ID[{}]获取会员信息]失败, 原因：{}", userId, msg.getResult());
                return null;
            }
            final MemberPhoneInfoVo user = msg.getData();
            if (user != null) {
                final MemberUserDTO userDTO = new MemberUserDTO();
                BeanUtils.copyProperties(user, userDTO);
                return userDTO;
            }
        } catch (Exception e) {
            log.error("调用Member系统[根据会员ID[{}]获取会员信息]失败", userId, e);
        }
        return null;
    }

    /**
     * 根据手机号查询会员信息
     *
     * @param mobile 手机号
     * @return 会员信息
     */
    public MemberUserDTO queryByMobile(final String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return null;
        }
        String key = String.format("%s:info:%s", getKeyPrefix(), mobile);
        MemberUserDTO memberUserDTO = getFromCache(key, MemberUserDTO.class);
        if (Objects.nonNull(memberUserDTO)) {
            return memberUserDTO;
        }
        try {
            Message<AccountInfoVo> accountMsg = accountApi.getAccountByPhone(mobile);
            isTrue(accountMsg.isSuccess(), String.format("调用Member系统失败: [根据会员手机号[%s]获取会员信息: %s]", mobile, accountMsg.getResult()));
            AccountInfoVo account = accountMsg.getData();
            if (account == null) {
                log.error("手机号[{}]不存在任何账号信息", mobile);
                return null;
            }
            final Message<MemberInfoVo> msg = memberApi.getMemberById(account.getMemberId());
            isTrue(msg.isSuccess(), String.format("调用Member系统失败: [根据会员手机号[%s]获取会员信息]", mobile));
            final MemberInfoVo user = msg.getData();
            if (Objects.isNull(user)) {
                return null;
            }
            final MemberUserDTO userDTO = new MemberUserDTO();
            BeanUtils.copyProperties(user, userDTO);
            userDTO.setLatestAccountId(account.getId());
            userDTO.setLatestAccountPhone(account.getPhone());
            setToCache(key, JSONObject.toJSONString(userDTO), 15);
            return userDTO;
        } catch (Exception e) {
            log.error("调用Member系统根据会员手机号[{}]获取会员信息失败", mobile, e);
        }
        return null;
    }

    @Nullable
    public UserAccountDTO accountById(final Long accountId) {
        if (accountId == null || accountId <= 0) {
            return null;
        }
        try {
            final Message<AccountInfoVo> msg = accountApi.getAccountById(accountId);
            if (!msg.isSuccess()) {
                log.warn("调用Member系统[根据账号ID[{}]获取账号信息]失败, 原因：{}", accountId, msg.getResult());
                return null;
            }
            final AccountInfoVo account = msg.getData();
            if (account != null) {
                final UserAccountDTO accountDTO = new UserAccountDTO();
                BeanUtils.copyProperties(account, accountDTO);
                return accountDTO;
            }
        } catch (Exception e) {
            log.error("调用Member系统[根据账号ID[{}]获取账号信息]失败", accountId, e);
        }
        return null;
    }

    /**
     * 查询手机号归属地
     *
     * @param mobile
     * @return
     */
    @Cacheable(cacheNames = "mobile:attribution", key = "#mobile", unless = "#result.isEmpty()")
    public String attribution(final String mobile) {
        String att = "";
        if (StringUtils.isBlank(mobile)) {
            return att;
        }
        att = CompletableFuture.supplyAsync(() -> {
                    MobileUtil.Result result = MobileUtil.attribution(mobile);
                    if (Objects.nonNull(result)) {
                        return result.getProvince().concat(" ").concat(result.getCity());
                    }
                    return "";
                }).applyToEither(CompletableFuture.supplyAsync(() -> {
                    try {
                        Message<MobileLocation> msg = functionApi.queryMobileLocation(mobile);
                        if (!msg.isSuccess()) {
                            return "";
                        }
                        MobileLocation data = msg.getData();
                        if (data != null) {
                            String province = data.getProvince();
                            String city = data.getCity();
                            return province.concat(" ").concat(city);
                        }
                    } catch (Exception err) {
                        log.warn("查询手机号归属地异常", err);
                    }
                    return "";
                }), r -> r)
                .join();
        return att.trim();
    }

    @Override
    public String getKeyPrefix() {
        return this.keyPrefix;
    }
}
