package cn.fw.shirasawa.sdk.api;

import cn.fw.data.base.domain.common.Message;
import cn.fw.shirasawa.sdk.param.*;
import cn.fw.shirasawa.sdk.result.SucessFollowRecordVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowApiService
 * @description : 跟进相关api
 * @date: 2020-08-14 17:51
 */
@FeignClient(value = "fw-shirasawa", path = "/api/shirasawa/follow")
public interface FollowApiService {

    /**
     * 保存会产生跟进的业务数据
     * 事故车的场景联系方式不能为空
     *
     * @param generateDTO generateDTO
     * @return
     */
    @PostMapping("/generate")
    Message<Void> generate(@Valid @RequestBody FollowGenerateDTO generateDTO);

    /**
     * 批量保存
     * 最多支持1000条
     * 事故车的场景联系方式不能为空
     *
     * @param paramList paramList
     * @return
     */
    @PostMapping("/generate_batch")
    Message<Void> generateBatch(@Valid @RequestBody CustomList<FollowGenerateDTO> paramList);


    /**
     * 变跟已有跟进任务的跟进人或跟进电话号码
     * <p>
     * 该任务必须是进行中的且除[任务跟进人、门店、电话号码]外其他信息未发生变化的
     * 如其他信息有变化的建议终止任务然后重新开始新的任务
     * </p>
     *
     * @param changeDTO
     * @return
     */
    @PostMapping("/change")
    Message<Void> taskChangeUser(@Valid @RequestBody FollowChangeUserDTO changeDTO);

    /**
     * 完成跟进任务
     *
     * @param taskCompleteDTO taskCompleteDTO
     * @return
     */
    @PostMapping("/complete/task")
    Message<Void> completeTask(@Valid @RequestBody TaskCompleteDTO taskCompleteDTO);

    /**
     * 终止跟进任务
     *
     * @param terminationDTO
     * @return
     */
    @PostMapping("/termination/task")
    Message<Void> terminationTask(@Valid @RequestBody TerminationDTO terminationDTO);

    /**
     * 批量终止任务
     * 最多支持1000条
     *
     * @param paramList
     * @return
     */
    @PostMapping("/termination_batch")
    Message<Void> terminationBatch(@Valid @RequestBody CustomList<TerminationDTO> paramList);

    /**
     * 查询用户剩余待办数
     *
     * @param userId  用户id
     * @param bizType 业态类型 {@link cn.fw.shirasawa.sdk.enums.BusinessTypeEnum}
     * @return
     */
    @GetMapping("/query_record_remaining")
    Message<Long> queryRecordRemaining(@RequestParam("userId") @NotNull(message = "userId不能为空！") Long userId,
                                       @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType);

    /**
     * 根据类型和业务id查询对应线索的下次跟进时间
     *
     * @param dataType 跟进类型 {@link cn.fw.shirasawa.sdk.enums.DataTypeEnum}
     * @param detailId 业务id类型
     * @param bizType  业态类型 {@link cn.fw.shirasawa.sdk.enums.BusinessTypeEnum}
     * @return
     */
    @GetMapping("/task_next_time")
    Message<Long> getTaskNextTime(@RequestParam("dataType") @NotNull(message = "类型不能为空") Integer dataType,
                                  @RequestParam("detailId") @NotBlank(message = "业务id不能为空") String detailId,
                                  @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType);

    /**
     * 完成跟进待办
     *
     * @param completeFollowDTO completeFollowDTO
     * @return
     * @description 当前仅支持售前crm部分特殊的需求
     */
    @PostMapping("/complete/record")
    Message<Void> completeFollow(@Valid @RequestBody CompleteFollowDTO completeFollowDTO);

    /**
     * 修改下次跟进待办推送时间
     *
     * @param dataType 跟进类型 {@link cn.fw.shirasawa.sdk.enums.DataTypeEnum}
     * @param detailId 业务id类型
     * @param nextTime 下次跟进时间
     * @param bizType  业态类型 {@link cn.fw.shirasawa.sdk.enums.BusinessTypeEnum}
     * @return
     * @description 当前仅支持售前crm部分特殊的需求
     */
    @PutMapping("/update/record/time")
    Message<Void> updateFollowNextTime(@RequestParam("dataType") @NotNull(message = "类型不能为空") Integer dataType,
                                       @RequestParam("detailId") @NotBlank(message = "业务id不能为空") String detailId,
                                       @RequestParam("nextTime") @NotNull(message = "下次跟进时间不能为空") Long nextTime,
                                       @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType
    );

    /**
     * 根据bizId、业务类型、门店id查询跟进数
     *
     * @param followCountReq 查询参数
     * @return
     * @since 1.1.2
     */
    @PostMapping("/count/biz_id")
    Message<Integer> getFollowCountByBizId(@Valid @RequestBody FollowCountReq followCountReq);

    /**
     * 生成下一次待办
     *
     * @param newRecordDTO
     * @return 结果
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @PostMapping("/create/record")
    Message<Boolean> createNextRecord(@Valid @RequestBody NewRecordDTO newRecordDTO);

    /**
     * 根据类型和档案id查询已完成的跟进记录
     *
     * @param bizType    跟进业务类型 1-售前  2-售后
     * @param customerId 档案id
     * @return
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @GetMapping("/getCompletedFollowRecord")
    Message<List<SucessFollowRecordVo>> getCompletedFollowRecord(@RequestParam("bizType") @NotNull(message = "业务类型不能为空！") Integer bizType,
                                                                 @RequestParam("customerId") @NotEmpty(message = "档案id不能为空！") List<Long> customerId);

    /**
     * 车辆进站终止推送待办
     * <p>目前只针对事故车的场景</p>
     * @param stopInShopDTO 参数
     * @return void
     */
    @PostMapping("/stop/for_in_shop")
    Message<Void> stopClueTodoForInShop(@Valid @RequestBody StopInShopDTO stopInShopDTO);
}