package cn.fw.shirasawa.sdk.param;

import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import lombok.*;
import org.hibernate.validator.constraints.ScriptAssert;

import javax.validation.constraints.Future;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.Duration;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

/**
 * @author : kurisu
 * @className : FollowOriginalParam
 * @description : 跟进原始数据
 * @date: 2020-08-14 17:41
 */
@Data
@EqualsAndHashCode
@ToString
@Builder
@AllArgsConstructor
@NoArgsConstructor
@ScriptAssert.List({
        @ScriptAssert(lang = "javascript", script = "_this.checkPlate()", message = "车牌号不能为空"),
        @ScriptAssert(lang = "javascript", script = "_this.checkVin()", message = "车架号不能为空"),
        @ScriptAssert(lang = "javascript", script = "_this.checkContact()", message = "联系方式不能为空"),
        @ScriptAssert(lang = "javascript", script = "_this.checkName()", message = "客户名称不能为空"),
        @ScriptAssert(lang = "javascript", script = "_this.checkCustomerId()", message = "档案id不能为空"),
        @ScriptAssert(lang = "javascript", script = "_this.checkFollowDuration()", message = "必须指定待办的跟进时长"),
        @ScriptAssert(lang = "javascript", script = "_this.checkActivityNo()", message = "活动编号不能为空"),
})
public class FollowGenerateDTO {
    /**
     * 数据类型
     */
    @NotNull(message = "数据类型不能为空")
    private DataTypeEnum type;
    /**
     * 业态类型
     */
    @NotNull(message = "业态类型不能为空")
    private BusinessTypeEnum businessType;
    /**
     * 档案id
     */
    private Long customerId;
    /**
     * 档案名称
     */
    private String customerName;
    /**
     * 会员id
     */
    private Long memberId;
    /**
     * 车牌号
     * 事故车的场景不能为空
     */
    private String plateNo;
    /**
     * 车架号
     */
    private String frameNo;
    /**
     * 联系方式[手机号]
     * 事故车的场景/或者没有会员号时 不能为空
     */
    private String contacts;
    /**
     * 业务id
     */
    @NotBlank(message = "业务id不能为空")
    private String detailId;
    /**
     * 待办系统展示参数
     */
    @NotEmpty(message = "待办展示不能为空")
    private Map<String, String> noteMap;
    /**
     * 数据产生时间
     */
    private Date generateTime;
    /**
     * 截止时间
     */
    @Future(message = "截止时间必须是未来时间")
    private Date deadline;
    /**
     * 跟进限制时长
     * 为空默认36h
     * 如果有特殊需求比如事故车则需要指定时长
     */
    private Duration followDuration;
    /**
     * 集团id
     */
    @NotNull(message = "集团id不能为空")
    private Long groupId;
    /**
     * 门店id
     */
    @NotNull(message = "门店id不能为空")
    private Long shopId;
    /**
     * 门店名称
     */
    private String shopName;
    /**
     * 跟进人员id
     */
    @NotNull(message = "跟进人员id不能为空")
    private Long userId;
    /**
     * 跟进人员名称
     */
    private String userName;
    /**
     * 业务Id
     * e.g: 活动编号/id
     * 特殊作用 比如按活动统计数据的区分
     */
    private String bizId;


    public boolean checkPlate() {
        if (DataTypeEnum.AC.equals(this.type)) {
            return this.plateNo != null && !this.plateNo.isEmpty();
        }
        return true;
    }

    public boolean checkName() {
        if (DataTypeEnum.RC.equals(this.type)) {
            return this.customerName != null && !this.customerName.isEmpty();
        }
        return true;
    }

    public boolean checkVin() {
        if (!DataTypeEnum.AC.equals(this.type) && BusinessTypeEnum.AS.equals(this.businessType)) {
            return this.frameNo != null && !this.frameNo.isEmpty();
        }
        return true;
    }

    public boolean checkContact() {
        if (DataTypeEnum.AC.equals(this.type)) {
            return this.plateNo != null &&
                    !this.plateNo.isEmpty() &&
                    this.contacts != null &&
                    !this.contacts.isEmpty();
        }
        if (Objects.isNull(this.memberId)) {
            return this.contacts != null && !this.contacts.isEmpty();
        }
        return true;
    }

    public boolean checkCustomerId() {
        if (BusinessTypeEnum.PS.equals(this.businessType) && !DataTypeEnum.RC.equals(this.type)) {
            return Objects.nonNull(this.customerId);
        }
        return true;
    }


    public boolean checkFollowDuration() {
        if (DataTypeEnum.AC.equals(this.type)) {
            return Objects.nonNull(this.followDuration);
        }
        return true;
    }

    public boolean checkActivityNo() {
        if (DataTypeEnum.AF.equals(this.type)) {
            return Objects.nonNull(this.bizId);
        }
        return true;
    }
}
