package cn.fw.shirasawa.server.controller.api;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.data.base.domain.common.Message;
import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.common.utils.MessageFormatUtil;
import cn.fw.shirasawa.domain.dto.FollowTaskCompleteDTO;
import cn.fw.shirasawa.domain.dto.FollowTerminationDTO;
import cn.fw.shirasawa.domain.dto.OriginalDataDTO;
import cn.fw.shirasawa.domain.enums.BizTypeEnum;
import cn.fw.shirasawa.domain.enums.FollowCloseReason;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.sdk.api.FollowApiService;
import cn.fw.shirasawa.sdk.enums.BusinessTypeEnum;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.sdk.enums.TerminationReason;
import cn.fw.shirasawa.sdk.param.*;
import cn.fw.shirasawa.sdk.result.SucessFollowRecordVo;
import cn.fw.shirasawa.service.bus.FollowExpansionService;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import cn.fw.shirasawa.service.bus.follow.OriginalDataBizService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 跟进系统api
 *
 * @author : kurisu
 * @className : FollowApiServiceImpl
 * @description : 跟进api
 * @date: 2020-08-14 17:53
 */
@Slf4j
@RestController
@RequestMapping("/api/shirasawa/follow")
public class FollowApiServiceImpl implements FollowApiService {
    @Value("${follow.max-size}")
    @Getter
    private int maxSize;

    private final OriginalDataBizService originalDataBizService;
    private final FollowBizService followBizService;
    private final FollowExpansionService followExpansionService;

    @Autowired
    public FollowApiServiceImpl(final OriginalDataBizService originalDataBizService,
                                final FollowBizService followBizService,
                                final FollowExpansionService followExpansionService) {
        this.originalDataBizService = originalDataBizService;
        this.followBizService = followBizService;
        this.followExpansionService = followExpansionService;
    }

    /**
     * 业务系统生成跟进任务
     *
     * @param generateDTO generateDTO
     * @return
     */
    @Override
    @PostMapping("/generate")
    @ControllerMethod("生成跟进任务")
    public Message<Void> generate(@Valid @RequestBody FollowGenerateDTO generateDTO) {
        OriginalDataDTO dataDTO = new OriginalDataDTO();
        BeanUtils.copyProperties(generateDTO, dataDTO);
        DataTypeEnum type = generateDTO.getType();
        dataDTO.setDataType(type.getValue());
        BizTypeEnum bizType = transferBizType(generateDTO.getBusinessType());
        BV.notNull(bizType, () -> "业态类型不正确");
        dataDTO.setBizType(bizType);
        originalDataBizService.saveOrigin(dataDTO);
        return success();
    }

    /**
     * 批量生成跟进任务
     *
     * @param paramList paramList
     * @return
     * @description 一次性最多支持1000条
     */
    @Override
    @PostMapping("/generate_batch")
    @ControllerMethod("批量生成跟进任务")
    public Message<Void> generateBatch(@Valid @RequestBody CustomList<FollowGenerateDTO> paramList) {
        BV.isNotEmpty(paramList, () -> "列表不能为空");
        BV.isTrue(paramList.size() <= getMaxSize(), () -> MessageFormatUtil.MessageFormatTransfer("最多支持一次性生成{0}条任务", getMaxSize()));
        List<OriginalDataDTO> list = new ArrayList<>();
        for (FollowGenerateDTO param : paramList) {
            OriginalDataDTO dataDTO = new OriginalDataDTO();
            BeanUtils.copyProperties(param, dataDTO);
            dataDTO.setDataType(param.getType().getValue());
            BizTypeEnum bizType = transferBizType(param.getBusinessType());
            BV.notNull(bizType, () -> "业态类型不正确");
            dataDTO.setBizType(bizType);
            list.add(dataDTO);
        }
        originalDataBizService.saveBatchOrigin(list);
        return success();
    }

    /**
     * 变跟已有跟进任务的跟进人或跟进电话号码
     *
     * @param changeDTO changeDTO
     * @return
     * @description 该任务必须是进行中的且除任务跟进人和门店外其他信息未发生变化的, 如其他信息有变化的建议终止任务然后重新开始新的任务
     */
    @Override
    @PostMapping("/change")
    @ControllerMethod("变跟跟进任务的跟进信息")
    public Message<Void> taskChangeUser(@Valid @RequestBody FollowChangeUserDTO changeDTO) {
        Date recommendNextTime = changeDTO.getRecommendNextTime();
        if (Objects.nonNull(recommendNextTime)) {
            boolean after = DateUtil.date2LocalDate(recommendNextTime).isAfter(LocalDate.now());
            BV.isTrue(after, () -> "推荐下次跟进时间必须是未来时间");
        }
        followBizService.taskChangeUser(changeDTO);
        return success();
    }


    /**
     * 完成跟进任务
     *
     * @param taskCompleteDTO taskCompleteDTO
     * @return
     */
    @Override
    @PostMapping("/complete/task")
    @ControllerMethod("完成跟进任务")
    public Message<Void> completeTask(@Valid @RequestBody TaskCompleteDTO taskCompleteDTO) {
        FollowTaskCompleteDTO dto = new FollowTaskCompleteDTO();
        BeanUtils.copyProperties(taskCompleteDTO, dto);
        dto.setFollowType(FollowTypeEnum.ofValue(taskCompleteDTO.getType().getValue()));
        BizTypeEnum bizType = transferBizType(taskCompleteDTO.getBusinessType());
        BV.notNull(bizType, () -> "业态类型不正确");
        dto.setBizType(bizType);
        followBizService.completeTask(dto);
        return success();
    }

    /**
     * 终止跟进任务
     *
     * @param terminationDTO terminationDTO
     * @return
     */
    @Override
    @PostMapping("/termination/task")
    @ControllerMethod("终止跟进任务")
    public Message<Void> terminationTask(@Valid @RequestBody TerminationDTO terminationDTO) {
        FollowTerminationDTO followTerminationDTO = new FollowTerminationDTO();
        BeanUtils.copyProperties(terminationDTO, followTerminationDTO);
        followTerminationDTO.setReason(transferEnum(terminationDTO.getReason()));
        followTerminationDTO.setFollowType(FollowTypeEnum.ofValue(terminationDTO.getType().getValue()));
        BizTypeEnum bizType = transferBizType(terminationDTO.getBusinessType());
        BV.notNull(bizType, () -> "业态类型不正确");
        followTerminationDTO.setBizType(bizType);
        followBizService.terminationTask(followTerminationDTO);
        return success();
    }

    /**
     * 批量终止任务
     *
     * @param paramList paramList
     * @return
     * @description 一次性最多支持1000条
     */
    @Override
    @PostMapping("/termination_batch")
    @ControllerMethod("批量终止跟进任务")
    public Message<Void> terminationBatch(@Valid @RequestBody CustomList<TerminationDTO> paramList) {
        BV.isNotEmpty(paramList, () -> "列表不能为空");
        BV.isTrue(paramList.size() <= getMaxSize(), () -> MessageFormatUtil.MessageFormatTransfer("最多支持一次性结束{0}条任务", getMaxSize()));
        List<FollowTerminationDTO> list = new ArrayList<>();
        for (TerminationDTO dto : paramList) {
            FollowTerminationDTO followTerminationDTO = new FollowTerminationDTO();
            BeanUtils.copyProperties(dto, followTerminationDTO);
            followTerminationDTO.setFollowType(FollowTypeEnum.ofValue(dto.getType().getValue()));
            BizTypeEnum bizType = transferBizType(dto.getBusinessType());
            BV.notNull(bizType, () -> "业态类型不正确");

            followTerminationDTO.setBizType(bizType);
            followTerminationDTO.setReason(transferEnum(dto.getReason()));
            list.add(followTerminationDTO);
        }
        followBizService.terminationTaskBatch(list);
        return success();
    }

    /**
     * 查询用户剩余待办数
     *
     * @param userId  用户id
     * @param bizType 业态类型 {@link cn.fw.shirasawa.sdk.enums.BusinessTypeEnum}
     * @return
     */
    @Override
    @GetMapping("/query_record_remaining")
    @ControllerMethod("查询用户剩余待办数")
    public Message<Long> queryRecordRemaining(@RequestParam("userId") @NotNull(message = "userId不能为空！") Long userId,
                                              @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType) {
        BizTypeEnum businessType = transferBizType(bizType);
        BV.notNull(businessType, () -> "业态类型不正确");
        return success(followBizService.queryOngoingRecord(userId, businessType));
    }

    /**
     * 根据类型和业务id查询对应线索的下次跟进时间
     *
     * @param dataType 跟进类型 {@link cn.fw.shirasawa.sdk.enums.DataTypeEnum}
     * @param detailId 业务id类型
     * @param bizType  业态类型 {@link cn.fw.shirasawa.sdk.enums.BusinessTypeEnum}
     * @return 下次跟进时间的时间戳（可能为空）
     */
    @Override
    @GetMapping("/task_next_time")
    @ControllerMethod("根据类型和业务id查询对应线索的下次跟进时间")
    public Message<Long> getTaskNextTime(@RequestParam("dataType") @NotNull(message = "类型不能为空") Integer dataType,
                                         @RequestParam("detailId") @NotBlank(message = "业务id不能为空") String detailId,
                                         @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType) {
        DataTypeEnum typeEnum = DataTypeEnum.ofValue(dataType);
        BV.notNull(typeEnum, () -> "跟进类型不正确");
        BizTypeEnum businessType = transferBizType(bizType);
        BV.notNull(businessType, () -> "业态类型不正确");
        return success(followBizService.queryTaskNextTime(Objects.requireNonNull(typeEnum), detailId, Objects.requireNonNull(businessType)));
    }

    /**
     * 完成跟进待办
     *
     * @param completeFollowDTO completeFollowDTO
     * @return
     * @description 当前仅支持售前crm部分特殊的需求
     */
    @Override
    @PostMapping("/complete/record")
    @ControllerMethod("完成跟进待办")
    public Message<Void> completeFollow(@Valid @RequestBody CompleteFollowDTO completeFollowDTO) {
        followBizService.completeRecord(completeFollowDTO.getRecordId(), completeFollowDTO.getUserId(), completeFollowDTO.getNextDto());
        return success();
    }


    /**
     * 修改下次跟进待办推送时间
     *
     * @param dataType 跟进类型 {@link cn.fw.shirasawa.sdk.enums.DataTypeEnum}
     * @param detailId 业务id类型
     * @param nextTime 下次跟进时间
     * @return
     * @description 当前仅支持售前crm部分特殊的需求
     */
    @Override
    @PutMapping("/update/record/time")
    @ControllerMethod("修改下次跟进待办推送时间")
    public Message<Void> updateFollowNextTime(@RequestParam("dataType") @NotNull(message = "类型不能为空") Integer dataType,
                                              @RequestParam("detailId") @NotBlank(message = "业务id不能为空") String detailId,
                                              @RequestParam("nextTime") @NotNull(message = "下次跟进时间不能为空") Long nextTime,
                                              @RequestParam("bizType") @NotNull(message = "业态类型不能为空！") Integer bizType
    ) {
        DataTypeEnum typeEnum = DataTypeEnum.ofValue(dataType);
        BV.notNull(typeEnum, () -> "跟进类型不正确");
        BizTypeEnum businessType = transferBizType(bizType);
        BV.notNull(businessType, () -> "业态类型不正确");
        LocalDate localDate = DateUtil.date2LocalDate(new Date(nextTime));
        BV.isTrue(LocalDate.now().isBefore(localDate), () -> "下次跟进时间必须时未来时");
        followBizService.updateFollowNextTime(Objects.requireNonNull(typeEnum), detailId, Objects.requireNonNull(businessType), localDate);
        return success();
    }

    /**
     * 根据bizId、业务类型、门店id查询跟进数
     *
     * @param followCountReq 查询参数
     * @return
     */
    @Override
    @PostMapping("/count/biz_id")
    @ControllerMethod("查询跟进数")
    public Message<Integer> getFollowCountByBizId(@Valid @RequestBody FollowCountReq followCountReq) {
        String bizId = followCountReq.getBizId();
        BizTypeEnum bizTypeEnum = transferBizType(followCountReq.getBizType());
        FollowTypeEnum followTypeEnum = FollowTypeEnum.ofValue(followCountReq.getType());
        List<Long> shopIds = followCountReq.getShopIds();
        int count = followBizService.queryFollowCountByBizId(bizId, followTypeEnum, bizTypeEnum, shopIds);
        return success(count);
    }

    /**
     * 生成下一次待办
     *
     * @param newRecordDTO newRecordDTO
     * @return 结果
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @Deprecated
    @Override
    @PostMapping("/create/record")
    @ControllerMethod("生成下一次待办")
    public Message<Boolean> createNextRecord(@Valid @RequestBody NewRecordDTO newRecordDTO) {
        return success(followBizService.createNewRecord(newRecordDTO));
    }

    /**
     * 根据类型和档案id查询已完成的跟进记录
     *
     * @param bizType    跟进业务类型 1-售前  2-售后
     * @param customerId 档案id
     * @return
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @Deprecated
    @Override
    @GetMapping("/getCompletedFollowRecord")
    @ControllerMethod("获取已完成的跟进记录")
    public Message<List<SucessFollowRecordVo>> getCompletedFollowRecord(@RequestParam("bizType") @NotNull(message = "业务类型不能为空！") Integer bizType,
                                                                        @RequestParam("customerId") @NotEmpty(message = "档案id不能为空！") List<Long> customerId) {
        return success(followBizService.getCompletedFollowRecord(bizType, customerId));
    }

    /**
     * 车辆进站终止推送待办
     * <p>目前只针对事故车的场景</p>
     *
     * @param stopInShopDTO 参数
     * @return void
     */
    @Override
    @PostMapping("/stop/for_in_shop")
    @ControllerMethod("车辆进站终止推送待办")
    public Message<Void> stopClueTodoForInShop(@Valid @RequestBody StopInShopDTO stopInShopDTO) {
        followExpansionService.stopClueTodoForInShop(stopInShopDTO);
        return success();
    }


    private FollowCloseReason transferEnum(Integer reasonCode) {
        TerminationReason reason = TerminationReason.ofValue(reasonCode);
        BV.notNull(reason, () -> "终止原因枚举值不能为空");
        FollowCloseReason closeReason = null;
        switch (Objects.requireNonNull(reason)) {
            case ABANDON:
                closeReason = FollowCloseReason.ABANDON;
                break;
            case ANOTHER:
                closeReason = FollowCloseReason.ANOTHER;
                break;
            case ROLE_CHANGE:
                closeReason = FollowCloseReason.ROLE_CHANGE;
                break;
            default:
                break;
        }
        return closeReason;
    }

    private BizTypeEnum transferBizType(Integer reasonCode) {
        BusinessTypeEnum businessTypeEnum = BusinessTypeEnum.ofValue(reasonCode);
        return transferBizType(businessTypeEnum);
    }

    private BizTypeEnum transferBizType(BusinessTypeEnum reason) {
        if (Objects.isNull(reason)) {
            return null;
        }
        switch (reason) {
            case AS:
                return BizTypeEnum.AS;
            case PS:
                return BizTypeEnum.PS;
        }
        return null;
    }
}
