package cn.fw.shirasawa.server.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.query.FollowRatioQuery;
import cn.fw.shirasawa.domain.vo.ViewDataVO;
import cn.fw.shirasawa.domain.vo.follow.*;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 跟进查询
 *
 * @author : kurisu
 * @version : 1.1
 * @className : FollowQueryController
 * @description : 跟进查询
 * @date : 2023-04-06 17:03
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/follow/query")
public class FollowQueryController {
    private final FollowBizService followBizService;

    @Autowired
    public FollowQueryController(final FollowBizService followBizService) {
        this.followBizService = followBizService;
    }

    /**
     * 查询跟进待办详情
     *
     * @param recordId 跟进记录id
     * @return
     */
    @IgnoreAuth
    @GetMapping("/todo/detail")
    @ControllerMethod("查询跟进待办详情")
    public Message<FollowDetailVO> todoDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        return success(followBizService.queryFollowDetail(recordId));
    }

    /**
     * 查询跟进战败审批详情
     *
     * @param recordId 跟进记录id
     * @return
     */
    @GetMapping("/approve/detail")
    @IgnoreAuth
    @ControllerMethod("查询跟进战败审批详情")
    public Message<FollowDetailVO> approveDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        FollowDetailVO detailVO = followBizService.queryApplyFollowDetail(recordId);
        return success(detailVO);
    }

    /**
     * 查询跟进待办历史记录
     *
     * @param recordId 跟进记录id
     * @return
     */
    @IgnoreAuth
    @GetMapping("/todo/history")
    @ControllerMethod("查询跟进待办历史记录")
    public Message<List<FollowRecordHistoryVO>> todoRecordHistory(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        return success(followBizService.todoRecordHistory(recordId));
    }

    /**
     * 查询短信模板
     *
     * @param currentUser 当前用户
     * @param type        跟进类型
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/sms/template")
    @IgnoreAuth
    @ControllerMethod("查询短信模板")
    public Message<String> getSmsTemplate(@CurrentUser LoginAuthBean currentUser, @NotNull(message = "跟进类型不能为空") final Integer type) {
        Message<String> message = success();
        message.setData(followBizService.smsTemplate(currentUser.getUserId(), type));
        return message;
    }

    /**
     * 查询档案跟进任务历史列表
     *
     * @param currentUser currentUser
     * @param customerId  档案id
     * @param bizType     业态类型
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/task/history/list")
    @IgnoreAuth
    @ControllerMethod("查询档案跟进线索历史列表")
    public Message<List<ClueHistoryVO>> clueHistory(@CurrentUser LoginAuthBean currentUser,
                                                    @NotNull(message = "档案id不能为空") final Long customerId,
                                                    final Integer bizType
    ) {
        List<ClueHistoryVO> list = followBizService.taskHistoryList(currentUser.getGroupId(), customerId, bizType);
        return success(list);
    }

    /**
     * 查询售后档案跟进线索历史列表
     *
     * @param currentUser currentUser
     * @param vin         vin
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/clue/after/history/list")
    @IgnoreAuth
    @ControllerMethod("查询vin跟进线索历史列表")
    public Message<List<ClueHistoryVO>> asClueHistory(@CurrentUser LoginAuthBean currentUser,
                                                      @NotBlank(message = "vin不能为空") final String vin,
                                                      final Integer type
    ) {
        FollowTypeEnum typeEnum = FollowTypeEnum.ofValue(type);
        List<ClueHistoryVO> list = followBizService.asTaskHistoryList(currentUser.getGroupId(), vin, typeEnum);
        return success(list);
    }

    /**
     * 查询跟进任务的跟进历史记录
     *
     * @param taskId 跟进任务id
     * @return
     */
    @GetMapping("/task/history/record")
    @IgnoreAuth
    @ControllerMethod("查询跟进线索的跟进历史记录")
    public Message<List<FollowRecordHistoryVO>> clueRecord(@NotNull(message = "跟进任务id不能为空") final Long taskId) {
        return success(followBizService.queryHistoryByTaskId(taskId));
    }

    /**
     * 查询客户的跟进历史记录
     *
     * @param customerId 客户档案用户id
     * @param bizType    业态类型
     * @return
     */
    @GetMapping("/customer/history/record")
    @IgnoreAuth
    @ControllerMethod("查询客户的跟进历史记录")
    public Message<List<FollowRecordHistoryVO>> customerRecord(@NotNull(message = "档案id不能为空") final Long customerId,
                                                               @NotNull(message = "业态类型不能为空") final Integer bizType,
                                                               final Integer type
    ) {
        return success(followBizService.queryHistoryByCustomer(customerId, bizType,  type));
    }

    /**
     * 查询车辆的跟进历史记录
     *
     * @param vin 客户vin
     * @return
     */
    @GetMapping("/vin/history/record")
    @IgnoreAuth
    @ControllerMethod("查询车辆的跟进历史记录")
    public Message<List<FollowRecordHistoryVO>> vinRecord(@CurrentUser LoginAuthBean currentUser, @NotBlank(message = "vin不能为空") final String vin) {
        return success(followBizService.queryHistoryByVin(vin, currentUser.getGroupId()));
    }

    /**
     * 查询跟进记录对应线索的跟进记录
     *
     * @param id 跟进待办id
     * @return
     */
    @GetMapping("/follow_record/log/detail")
    @IgnoreAuth
    @ControllerMethod("查询跟进记录池详情")
    public Message<List<FollowRecordHistoryVO>> recordPoolDetail(@NotNull(message = "跟进待办id不能为空") final Long id) {
        return success(followBizService.recordPoolDetail(id));
    }

    /**
     * 根据recordId获取跟进记录详细信息
     *
     * @param recordId 跟进id
     * @return 跟进记录详情
     */
    @GetMapping("/followRecord/history/record")
    @IgnoreAuth
    @ControllerMethod("查询跟进记录详情")
    public Message<List<DateFollowRecordHistoryVO>> followRecordDetail(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        List<DateFollowRecordHistoryVO> detailVoList = followBizService.followRecordDetail(recordId);
        return success(detailVoList);
    }

    /**
     * 根据recordId获取进行中的其他跟进线索
     *
     * @param recordId 跟进id
     * @return 跟进记录详情
     */
    @GetMapping("/other/ongoing/follow")
    @IgnoreAuth
    @ControllerMethod("查询当期进行中的其他跟进线索")
    public Message<List<FollowTaskSimpleVO>> otherRecord(@NotNull(message = "跟进记录id不能为空") final Long recordId) {
        return success(followBizService.queryUserOtherTask(recordId));
    }

    /**
     * 查询跟进数量占比 【活动报表使用】
     *
     * @return : 列表
     * @since : 1.1.2
     */
    @GetMapping("/todo/ratio")
    @IgnoreAuth
    @ControllerMethod("查询跟进数量占比")
    public Message<List<ViewDataVO>> queryFollowRatio(@Valid FollowRatioQuery query) {
        return success(followBizService.queryFollowRatio(query));
    }
}
