package cn.fw.shirasawa.server.controller.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.common.cache.locker.FLock;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.enums.FollowStatusEnum;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.rpc.backlog.TodoRpcService;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import cn.fw.shirasawa.service.bus.follow.FollowExecuteService;
import cn.fw.shirasawa.service.data.FollowRecordService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.lang.NonNull;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowRecordTask
 * @description : 跟进任务定时任务
 * @date: 2020-08-25 16:56
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class FollowRecordTask {
    private final FollowRecordService followRecordService;
    private final FollowBizService followBizService;
    private final TodoRpcService todoRpcService;
    private final DistributedLocker distributedLocker;
    private final FollowExecuteService followExecuteService;

    @Value("${spring.cache.custom.global-prefix}:task")
    @Getter
    private String prefix;

    @Autowired
    public FollowRecordTask(final FollowRecordService followRecordService,
                            final FollowBizService followBizService,
                            final TodoRpcService todoRpcService,
                            final DistributedLocker distributedLocker,
                            final FollowExecuteService followExecuteService) {
        this.followRecordService = followRecordService;
        this.followBizService = followBizService;
        this.todoRpcService = todoRpcService;
        this.distributedLocker = distributedLocker;
        this.followExecuteService = followExecuteService;
    }

    /**
     * 处理跟进逾期
     */
    @Scheduled(initialDelay = 1000 * 15, fixedRate = 1000 * 5)
    public void endTaskRecord() {
        final String cacheName = getCacheName("end-task");
        RLock lock = distributedLocker.lock(cacheName);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                    .le(FollowRecord::getDeadline, LocalDateTime.now())
                    .eq(FollowRecord::getYn, Boolean.TRUE)
                    .last("limit 0, 300")
            );
            if (CollectionUtils.isEmpty(list)) {
                return;
            }
            for (FollowRecord record : list) {
                followBizService.overdueProcessing(record);
            }
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }

    /**
     * 普通待办指定时间才推送
     * 推送到待办系统
     */
    @Scheduled(cron = "0/8 * 9-18 * * *")
    public void push2NorTodo() {
        final String cacheName = getCacheName("push-task:normal");
        RLock lock = distributedLocker.lock(cacheName);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                    .eq(FollowRecord::getAddTodo, Boolean.FALSE)
                    .notIn(FollowRecord::getType, FollowTypeEnum.AC, FollowTypeEnum.SF, FollowTypeEnum.PL)
                    .le(FollowRecord::getPlanTime, LocalDateTime.now())
                    .eq(FollowRecord::getYn, Boolean.TRUE)
                    .orderByAsc(FollowRecord::getDeadline)
                    .last("limit 0,500")
            );
            followExecuteService.execute(list);
        } catch (Exception e) {
            log.warn("定时推送普通待办失败", e);
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }

    /**
     * 事故车待办随时推送
     * 推送到待办系统
     */
    @Scheduled(initialDelay = 1500, fixedRate = 1000 * 3)
    public void push2AccTodo() {
        final String cacheName = getCacheName("push-task:acc");
        RLock lock = distributedLocker.lock(cacheName);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                    .eq(FollowRecord::getAddTodo, Boolean.FALSE)
                    .eq(FollowRecord::getYn, Boolean.TRUE)
                    .in(FollowRecord::getType, FollowTypeEnum.AC, FollowTypeEnum.SF, FollowTypeEnum.PL)
                    .le(FollowRecord::getPlanTime, LocalDateTime.now())
                    .last("limit 0,50")
            );
            followExecuteService.execute(list);
        } catch (Exception e) {
            log.warn("定时推送及时待办失败", e);
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }


    /**
     * 完成待办的重试
     */
    @Scheduled(initialDelay = 1500, fixedRate = 15 * 1000)
    public void retryCompleteTodoItem() {
        final String cacheName = getCacheName("retry:complete");
        RLock lock = distributedLocker.lock(cacheName);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            String key = todoRpcService.generateKey(TodoRpcService.TodoOperationEnum.COMPLETE);
            Collection<BackLogItemDTO> all = todoRpcService.getAllFromCache(key);
            all.forEach(todoRpcService::complete);
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }

    /**
     * 取消待办的重试
     */
    @Scheduled(initialDelay = 1000, fixedRate = 10 * 1000)
    public void retryCancelTodoItem() {
        final String cacheName = getCacheName("retry:cancel");
        FLock lock = distributedLocker.lock(cacheName);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            String key = todoRpcService.generateKey(TodoRpcService.TodoOperationEnum.CANCEL);
            Collection<BackLogItemDTO> all = todoRpcService.getAllFromCache(key);
            all.forEach(todoRpcService::cancel);
        } finally {
            if (locked) {
                distributedLocker.unlock(lock);
            }
        }
    }

    private String getCacheName(@NonNull String name) {
        return getPrefix() + ":" + name;
    }

}
