package cn.fw.shirasawa.server.controller.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.follow.TaskNextPlan;
import cn.fw.shirasawa.domain.enums.TaskStateEnum;
import cn.fw.shirasawa.service.bus.follow.FollowBizService;
import cn.fw.shirasawa.service.data.FollowTaskService;
import cn.fw.shirasawa.service.data.TaskNextPlanService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowTask
 * @description : 跟进任务处理
 * @date: 2020-08-24 16:32
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class FollowTaskDealTask {
    private final FollowTaskService followTaskService;
    private final FollowBizService followBizService;
    private final TaskNextPlanService taskNextPlanService;
    private final DistributedLocker distributedLocker;
    @Value("${spring.cache.custom.global-prefix}:clue")
    @Getter
    private String prefix;

    @Autowired
    public FollowTaskDealTask(final FollowTaskService followTaskService,
                              final FollowBizService followBizService,
                              final TaskNextPlanService taskNextPlanService,
                              final DistributedLocker distributedLocker) {
        this.followTaskService = followTaskService;
        this.followBizService = followBizService;
        this.taskNextPlanService = taskNextPlanService;
        this.distributedLocker = distributedLocker;
    }

    /**
     * 逾期结束任务
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void endTask() {
        String key = getCacheName("end");
        RLock lock = distributedLocker.lock(key);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                    .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                    .le(FollowTask::getDeadline, LocalDateTime.now())
            );
            if (CollectionUtils.isEmpty(list)) {
                return;
            }
            for (FollowTask task : list) {
                followBizService.endTask(task);
            }
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }

    /**
     * 生成下次待办
     */
    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 90)
    public void createNextRecord() {
        String key = getCacheName("gen-next-record");
        RLock lock = distributedLocker.lock(key);
        boolean locked = lock.isLocked();
        if (!locked) {
            return;
        }
        try {
            List<TaskNextPlan> list = taskNextPlanService.list(Wrappers.<TaskNextPlan>lambdaQuery()
                    .eq(TaskNextPlan::getEffective, Boolean.TRUE)
                    .le(TaskNextPlan::getPlanTime, LocalDate.now())
                    .last(" limit 500")
            );
            if (CollectionUtils.isEmpty(list)) {
                return;
            }
            for (TaskNextPlan r : list) {
                followBizService.createNewRecord(r.getTaskId(), r.getPlanTime());
            }
        } finally {
            if (lock.isHeldByCurrentThread()) {
                distributedLocker.unlock(lock);
            }
        }
    }

    private String getCacheName(String intent) {
        return getPrefix() + ":" + intent;
    }
}
