package cn.fw.shirasawa.service.bus;

import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.component.RecordResultProducer;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.other.ApproveRecord;
import cn.fw.shirasawa.domain.enums.ApproveStateEnum;
import cn.fw.shirasawa.domain.enums.ApproveTypeEnum;
import cn.fw.shirasawa.domain.enums.CallStateEnum;
import cn.fw.shirasawa.domain.enums.FollowCloseReason;
import cn.fw.shirasawa.rpc.flow.FlowApproveRpc;
import cn.fw.shirasawa.sdk.enums.CallEfficientEnum;
import cn.fw.shirasawa.sdk.mq.FollowResultDTO;
import cn.fw.shirasawa.service.data.ApproveRecordService;
import cn.fw.shirasawa.service.data.FollowRecordService;
import cn.fw.shirasawa.service.data.FollowTaskService;
import cn.fw.shirasawa.service.data.SecretReportHistoryService;
import cn.fw.shirasawa.service.event.CancelApproveEvent;
import cn.fw.shirasawa.service.event.RecordCompleteEvent;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Objects;

/**
 * @author : kurisu
 * @className : CustomEventListener
 * @description : 事件监听器
 * @date: 2020-08-18 16:28
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class CustomEventListener {
    private final FollowTaskService followTaskService;
    private final ApproveRecordService approveRecordService;
    private final FlowApproveRpc flowApproveRpc;
    private final FollowRecordService followRecordService;
    private final SecretReportHistoryService secretReportHistoryService;
    private final RecordResultProducer recordResultProducer;

    /**
     * 取消审批
     *
     * @param event
     */
    @EventListener(CancelApproveEvent.class)
    public void cancelApprove(final CancelApproveEvent event) {
        Long taskId = event.getTaskId();
        ApproveTypeEnum typeEnum = event.getTypeEnum();
        ApproveRecord approveRecord = approveRecordService.getOne(Wrappers.<ApproveRecord>lambdaQuery()
                .eq(ApproveRecord::getDataId, taskId)
                .eq(ApproveRecord::getType, typeEnum)
                .eq(ApproveRecord::getState, ApproveStateEnum.WAIT)
                .last("limit 1")
        );
        if (Objects.isNull(approveRecord)) {
            return;
        }
        flowApproveRpc.cancel(approveRecord.getOrderNo());
        approveRecord.setState(ApproveStateEnum.CANCELED);
        approveRecordService.updateById(approveRecord);
    }

    /**
     * 待办处理的时间
     */
    @EventListener(RecordCompleteEvent.class)
    public void onRecordComplete(final RecordCompleteEvent t) {
        try {
            final FollowRecord record = t.getRecord();
            FollowTask task = t.getTask();
            if (Objects.isNull(task)) {
                Long taskId = record.getTaskId();
                task = followTaskService.getById(taskId);
            }
            if (Objects.isNull(task)) {
                return;
            }
            final Long recordId = record.getId();
            final FollowResultDTO dto = new FollowResultDTO();
            fillBean(dto, record, task);
            long remaining = followRecordService.recordRemaining(record.getUserId(), record.getBizType(), recordId);
            dto.setRemaining(remaining);

            if (!CallStateEnum.WAIT_DIAL.equals(record.getCalled())) {
                boolean validCall = CallStateEnum.EFFECTIVE_DIAL.equals(record.getCalled());
                CallEfficientEnum efficientEnum = validCall ? CallEfficientEnum.VALID : CallEfficientEnum.INVALID;
                dto.setCallEfficient(efficientEnum.getValue());
            }
            recordResultProducer.send(dto);
        } catch (Exception ex) {
            log.error("RecordCompleteEvent处理失败", ex);
        }
    }

    private void fillBean(final FollowResultDTO dto, final FollowRecord record, FollowTask task) {
        dto.setCustomerId(record.getCustomerId());
        dto.setRecordId(record.getId());
        dto.setType(record.getType().getValue());
        dto.setDetailId(task.getDetailId());
        dto.setBizId(task.getBizId());
        dto.setBusinessType(task.getBizType().getValue());
        dto.setUserId(record.getUserId());
        dto.setUserName(record.getUserName());
        dto.setShopId(record.getShopId());
        dto.setGroupId(record.getGroupId());
        dto.setOverdue(FollowCloseReason.OVERDUE.equals(record.getReason()));
        dto.setResultTime(DateUtil.toDate(record.getCloseTime()));
        dto.setCallEfficient(CallEfficientEnum.INAPPLICABLE.getValue());
    }
}
