package cn.fw.shirasawa.service.bus.follow;

import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.shirasawa.common.constant.RoleCode;
import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.component.ApproveDefeatFollowProducer;
import cn.fw.shirasawa.domain.db.follow.*;
import cn.fw.shirasawa.domain.db.other.ApproveRecord;
import cn.fw.shirasawa.domain.db.other.OriginalData;
import cn.fw.shirasawa.domain.db.other.SecretReportHistory;
import cn.fw.shirasawa.domain.dto.*;
import cn.fw.shirasawa.domain.enums.*;
import cn.fw.shirasawa.domain.query.FollowRatioQuery;
import cn.fw.shirasawa.domain.vo.ViewDataVO;
import cn.fw.shirasawa.domain.vo.follow.*;
import cn.fw.shirasawa.rpc.backlog.TodoRpcService;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.rpc.ehr.EhrRpcService;
import cn.fw.shirasawa.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.shirasawa.rpc.erp.UserService;
import cn.fw.shirasawa.rpc.erp.dto.UserInfoDTO;
import cn.fw.shirasawa.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.shirasawa.rpc.flow.FlowApproveRpc;
import cn.fw.shirasawa.rpc.flow.dto.FlowDto;
import cn.fw.shirasawa.rpc.member.MemberRpcService;
import cn.fw.shirasawa.rpc.member.dto.MemberUserDTO;
import cn.fw.shirasawa.rpc.oop.OopService;
import cn.fw.shirasawa.rpc.oop.dto.ShopDTO;
import cn.fw.shirasawa.rpc.sms.SmsRpcService;
import cn.fw.shirasawa.sdk.enums.DataTypeEnum;
import cn.fw.shirasawa.sdk.mq.FollowApproveDTO;
import cn.fw.shirasawa.sdk.param.CompleteFollowDTO;
import cn.fw.shirasawa.sdk.param.FollowChangeUserDTO;
import cn.fw.shirasawa.sdk.param.NewRecordDTO;
import cn.fw.shirasawa.sdk.result.SucessFollowRecordVo;
import cn.fw.shirasawa.service.bus.follow.strategy.FollowStrategy;
import cn.fw.shirasawa.service.data.*;
import cn.hutool.core.collection.ListUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : FollowBizService
 * @description : 跟进业务服务
 * @date: 2020-08-14 17:27
 */
@Slf4j
@Service
public class FollowBizService {
    /**
     * 跟进处理器map
     */
    private final Map<FollowTypeEnum, FollowStrategy> followMap;
    private final FollowRecordService followRecordService;
    private final FollowTaskService followTaskService;
    private final UserService userService;
    private final SmsRpcService smsRpcService;
    private final EhrRpcService ehrRpcService;
    private final OopService oopService;
    private final ApproveRecordService approveRecordService;
    private final FlowApproveRpc flowApproveRpc;
    private final FollowRecordLogService followRecordLogService;
    private final SecretReportHistoryService secretReportHistoryService;
    private final MemberRpcService memberRpcService;
    private final ApproveDefeatFollowProducer approveDefeatFollowProducer;
    private final StringRedisTemplate stringRedisTemplate;
    private final TaskChangeLogService taskChangeLogService;
    private final OriginalDataService originalDataService;
    private final TaskNextPlanService taskNextPlanService;
    private final TodoRpcService todoRpcService;

    @Value("${follow.pno}")
    @Getter
    private String preFlowNo;

    @Value("${follow.ano}")
    @Getter
    private String afterFlowNo;

    @Value("${spring.cache.custom.global-prefix}:follow-record")
    @Getter
    private String keyPrefix;

    @Value("${follow.retry}")
    private Boolean retry;

    @Autowired
    public FollowBizService(final List<FollowStrategy> followStrategyList,
                            final FollowRecordService followRecordService,
                            final FollowTaskService followTaskService,
                            final UserService userService,
                            final SmsRpcService smsRpcService,
                            final EhrRpcService ehrRpcService,
                            final OopService oopService,
                            final ApproveRecordService approveRecordService,
                            final FlowApproveRpc flowApproveRpc,
                            final FollowRecordLogService followRecordLogService,
                            final SecretReportHistoryService secretReportHistoryService,
                            final MemberRpcService memberRpcService,
                            final ApproveDefeatFollowProducer approveDefeatFollowProducer,
                            final StringRedisTemplate stringRedisTemplate,
                            final TaskChangeLogService taskChangeLogService,
                            final OriginalDataService originalDataService,
                            final TaskNextPlanService taskNextPlanService,
                            final TodoRpcService todoRpcService) {
        this.followMap = followStrategyList.stream().collect(Collectors.toMap(FollowStrategy::getFollowType, v -> v));
        this.followRecordService = followRecordService;
        this.followTaskService = followTaskService;
        this.userService = userService;
        this.smsRpcService = smsRpcService;
        this.ehrRpcService = ehrRpcService;
        this.oopService = oopService;
        this.approveRecordService = approveRecordService;
        this.flowApproveRpc = flowApproveRpc;
        this.followRecordLogService = followRecordLogService;
        this.secretReportHistoryService = secretReportHistoryService;
        this.memberRpcService = memberRpcService;
        this.approveDefeatFollowProducer = approveDefeatFollowProducer;
        this.stringRedisTemplate = stringRedisTemplate;
        this.taskChangeLogService = taskChangeLogService;
        this.originalDataService = originalDataService;
        this.taskNextPlanService = taskNextPlanService;
        this.todoRpcService = todoRpcService;
    }

    /**
     * 元数据生成线索
     *
     * @param originalData
     */
    public boolean origin2task(OriginalData originalData) {
        if (Objects.isNull(originalData)) {
            return true;
        }
        FollowTypeEnum type = originalData.getType();
        FollowStrategy strategy = followMap.get(type);
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            return strategy.origin2task(originalData);
        } catch (Exception exception) {
            log.error("处理元数据失败 data: [{}]", originalData, exception);
            return !Boolean.TRUE.equals(this.retry);
        }
    }

    /**
     * 结束跟进任务
     *
     * @param task
     */
    public void endTask(FollowTask task) {
        try {
            FollowStrategy strategy = followMap.get(task.getType());
            Assert.notNull(strategy, "strategy cannot be null");

            strategy.endTask(task, false);
        } catch (Exception e) {
            log.error("结束任务失败：taskId[{}]", task.getId(), e);
        }
    }


    /**
     * 跟进待办逾期
     *
     * @param record
     */
    public void overdueProcessing(FollowRecord record) {
        if (Objects.isNull(record)) {
            return;
        }
        FollowTypeEnum type = record.getType();
        FollowStrategy strategy = followMap.get(type);
        Assert.notNull(strategy, "strategy cannot be null");
        try {
            strategy.overdue(record);
        } catch (Exception exception) {
            log.error("处理逾期数据失败 data: [{}]", record, exception);
        }
    }

    /**
     * 完成跟进待办
     *
     * @param dto dto
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'complete:' + #dto.recordId", message = "正在完成跟进，请勿重复操作")
    public void completeRecord(RecordCompleteDTO dto) {
        Long recordId = dto.getRecordId();
        FollowRecord record = followRecordService.getById(recordId);
        if (Objects.isNull(record) || Boolean.FALSE.equals(record.getYn())) {
            return;
        }
        FollowStrategy strategy = followMap.get(record.getType());
        strategy.complete(record, record.getUserId(), dto.getNextDto());
    }

    /**
     * 完成跟进待办
     * api only
     *
     * @param recordId
     * @param userId
     * @param nextInfoList
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'complete:' + #recordId", message = "正在完成跟进，请勿重复操作")
    public void completeRecord(Long recordId, Long userId, List<CompleteFollowDTO.NextInfo> nextInfoList) {
        FollowRecord record = followRecordService.getById(recordId);
        if (Objects.isNull(record) || Boolean.FALSE.equals(record.getYn())) {
            return;
        }
        List<RecordCompleteDTO.NextDTO> list = null;
        if (!CollectionUtils.isEmpty(nextInfoList)) {
            list = nextInfoList.stream().map(r -> {
                RecordCompleteDTO.NextDTO nextDTO = new RecordCompleteDTO.NextDTO();
                nextDTO.setNextDate(DateUtil.date2LocalDate(r.getNextDate()));
                nextDTO.setTaskId(r.getTaskId());
                return nextDTO;
            }).collect(Collectors.toList());
        }
        FollowStrategy strategy = followMap.get(record.getType());
        strategy.complete(record, userId, list);
    }

    /**
     * 更新下次跟进时间
     *
     * @param dataType     dataType
     * @param detailId     detailId
     * @param nextTime     nextTime
     * @param businessType businessType
     */
    public void updateFollowNextTime(DataTypeEnum dataType, String detailId, BizTypeEnum businessType, LocalDate nextTime) {
        BV.notNull(nextTime, () -> "下次跟进时间时必要参数");
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                        .eq(FollowTask::getDetailId, detailId)
                        .eq(FollowTask::getBizType, businessType.getValue())
                        .eq(FollowTask::getType, dataType.getValue())
                , Boolean.FALSE
        );
        if (Objects.isNull(task)) {
            return;
        }
        Long taskId = task.getId();
        TaskNextPlan plan = taskNextPlanService.queryByTaskId(taskId);
        if (Objects.isNull(plan)) {
            if (!task.getDeadline().isAfter(nextTime.atStartOfDay())) {
                return;
            }
            plan = new TaskNextPlan()
                    .setTaskId(taskId)
                    .setEffective(Boolean.TRUE);
        }
        plan.setPlanTime(nextTime);
        taskNextPlanService.saveOrUpdate(plan);
    }


    /**
     * 通过跟进类型和业务id查询下次跟进时间
     *
     * @param dataType     跟进类型
     * @param detailId     业务id
     * @param businessType businessType
     * @return 下次跟进时间的时间戳
     */
    @Nullable
    public Long queryTaskNextTime(DataTypeEnum dataType, String detailId, BizTypeEnum businessType) {
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                        .eq(FollowTask::getDetailId, detailId)
                        .eq(FollowTask::getType, dataType.getValue())
                        .eq(FollowTask::getBizType, businessType.getValue())
                , Boolean.FALSE
        );
        if (Objects.isNull(task)) {
            return null;
        }
        Long taskId = task.getId();
        TaskNextPlan plan = taskNextPlanService.queryByTaskId(taskId);
        if (Objects.isNull(plan)) {
            return null;
        }
        return DateUtil.toDate(plan.getPlanTime()).getTime();
    }

    /**
     * 查询对应bizID的跟进数
     *
     * @param bizId       业务编号
     * @param typeEnum    跟进类型
     * @param bizTypeEnum 业态类型
     * @param shopIds     门店id
     * @return 跟进数
     */
    public int queryFollowCountByBizId(String bizId, FollowTypeEnum typeEnum, BizTypeEnum bizTypeEnum, List<Long> shopIds) {
        return followRecordService.count(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getBizId, bizId)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .in(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING, FollowStatusEnum.CLOSED)
                .eq(Objects.nonNull(bizTypeEnum), FollowRecord::getBizType, bizTypeEnum)
                .eq(Objects.nonNull(typeEnum), FollowRecord::getType, typeEnum)
                .in(!CollectionUtils.isEmpty(shopIds), FollowRecord::getShopId, shopIds)
        );
    }

    /**
     * 生成新待办
     *
     * @param newRecordDTO
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'create:' + #newRecordDTO.userId + ':' + #newRecordDTO.customerId", message = "请勿重复操作")
    @Transactional(rollbackFor = Exception.class)
    public Boolean createNewRecord(NewRecordDTO newRecordDTO) {
        // fixme 删除
        return true;
    }

    /**
     * 生成新待办
     *
     * @param taskId
     * @param nextDate
     */
    public void createNewRecord(Long taskId, LocalDate nextDate) {
        try {
            FollowTask task = followTaskService.getById(taskId);
            if (Objects.isNull(task) || !TaskStateEnum.ONGOING.equals(task.getState())) {
                taskNextPlanService.stopFollow(taskId);
                return;
            }
            FollowStrategy strategy = followMap.get(task.getType());
            strategy.genNextRecord(task, nextDate);
        } catch (Exception e) {
            log.error("生成下次待办失败 taskId: {}", taskId, e);
        }
    }

    /**
     * 上传跟进附件
     *
     * @param dto
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'upload:' + #dto.recordId", message = "附件上传中，请勿重复操作")
    public void uploadAtt(FollowAttachmentDTO dto, Long userId) {
        FollowRecord record = followRecordService.getById(dto.getRecordId());
        if (Objects.isNull(record) || !record.getYn()) {
            return;
        }
        FollowStrategy strategy = followMap.get(record.getType());
        strategy.uploadAtt(dto, record, userId);
    }

    /**
     * 完成跟进任务
     *
     * @param taskCompleteDTO
     */
    public void completeTask(FollowTaskCompleteDTO taskCompleteDTO) {
        FollowStrategy strategy = followMap.get(taskCompleteDTO.getFollowType());
        strategy.completeTask(taskCompleteDTO);
    }

    /**
     * 终止跟进
     *
     * @param terminationDTO
     */
    public void terminationTask(FollowTerminationDTO terminationDTO) {
        FollowStrategy strategy = followMap.get(terminationDTO.getFollowType());
        strategy.terminationTaskPlan(terminationDTO);
    }

    /**
     * 批量终止任务
     *
     * @param list
     */
    public void terminationTaskBatch(List<FollowTerminationDTO> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        FollowTypeEnum followType = list.get(0).getFollowType();
        FollowStrategy strategy = followMap.get(followType);
        for (FollowTerminationDTO followTerminationDTO : list) {
            strategy.terminationTaskPlan(followTerminationDTO);
        }
    }

    /**
     * 查询跟进详情
     *
     * @param recordId 跟进待办id
     * @return 跟进详情
     */
    public FollowDetailVO queryFollowDetail(Long recordId) {
        FollowRecord followRecord = followRecordService.getById(recordId);
        Boolean recordYn = Optional.ofNullable(followRecord).map(FollowRecord::getYn).orElse(Boolean.FALSE);
        BV.isTrue(recordYn, () -> "跟进待办不存在");
        FollowTypeEnum type = Objects.requireNonNull(followRecord).getType();
        FollowStrategy strategy = followMap.get(type);
        return strategy.queryDetail(followRecord);
    }

    /**
     * 查询跟进详情(审批用)
     *
     * @param recordId 跟进待办id
     * @return 跟进详情
     */
    public FollowDetailVO queryApplyFollowDetail(Long recordId) {
        FollowDetailVO vo = queryFollowDetail(recordId);
        ApproveRecord approveRecord = approveRecordService.getOne(Wrappers.<ApproveRecord>lambdaQuery()
                .eq(ApproveRecord::getDataId, vo.getTaskId())
                .eq(ApproveRecord::getUserId, vo.getUserId())
                .eq(ApproveRecord::getType, ApproveTypeEnum.FOLLOW_DEFEAT)
                .last(" limit 1"));
        if (Objects.nonNull(approveRecord)) {
            vo.setReason(approveRecord.getReason());
        }
        return vo;
    }

    /**
     * 查询跟进历史
     *
     * @param recordId 跟进待办id
     * @return 跟进详情
     */
    public List<FollowRecordHistoryVO> todoRecordHistory(Long recordId) {
        BV.notNull(recordId, () -> "跟进记录id不能为空");
        FollowRecord followRecord = followRecordService.getById(recordId);
        BV.notNull(followRecord, () -> "跟进待办不存在");
        BV.isTrue(followRecord.getYn(), () -> "跟进待办不存在");
        FollowTypeEnum type = followRecord.getType();
        FollowStrategy strategy = followMap.get(type);
        return strategy.queryHistoryByRecord(followRecord);
    }

    /**
     * 查询跟进详情
     *
     * @param taskId 跟进任务id
     * @return 跟进详情
     */
    public List<FollowRecordHistoryVO> todoHistoryTask(Long taskId) {
        FollowTask task = followTaskService.getById(taskId);
        BV.notNull(task, () -> "跟进任务不存在");
        FollowTypeEnum type = task.getType();
        FollowStrategy strategy = followMap.get(type);
        return strategy.queryHistoryByTask(task);
    }

    /**
     * 查询短信模板
     *
     * @param type
     * @return
     */
    public String smsTemplate(Long userId, Integer type) {
        FollowTypeEnum followType = FollowTypeEnum.ofValue(type);
        BV.notNull(followType, () -> "跟进类型不正确");
        StaffInfoDTO staffInfo = ehrRpcService.queryStaffInfo(userId);
        BV.notNull(staffInfo, () -> "员工信息获取失败");
        List<UserRoleDataRangeDTO> range = null;
        if (FollowTypeEnum.AC.equals(followType)) {
            range = userService.getUserRoleDataRange(userId, RoleCode.SGCGJ);
        }
        if (FollowTypeEnum.FM.equals(followType) || FollowTypeEnum.RM.equals(followType)) {
            range = userService.getUserRoleDataRange(userId, RoleCode.FWGW);
        }
        if (FollowTypeEnum.IR.equals(followType)) {
            range = userService.getUserRoleDataRange(userId, RoleCode.XBGJ);
        }
        BV.isNotEmpty(range, () -> "门店信息获取失败");
        Long shopId = Objects.requireNonNull(range).get(0).getRangeValue();
        String shopName = range.get(0).getRangeName();
        return smsRpcService.getSmsTemplate(shopId, followType) + "\n" + "门店：" + shopName + "专属顾问：" + staffInfo.getName() + ";" + "手机号：" + staffInfo.getMobile() + "; 微信同号。";
    }

    /**
     * 变跟任务跟进人
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public void taskChangeUser(FollowChangeUserDTO dto) {
        if (Objects.isNull(dto)) {
            return;
        }
        boolean emptyMobile = cn.fw.shirasawa.common.utils.StringUtils.isEmpty(dto.getMobile());
        if (emptyMobile) {
            BV.isTrue(Objects.nonNull(dto.getUserId()) && Objects.nonNull(dto.getShopId()), () -> "跟进人和跟进门店不能为空");
        }
        final String detailId = dto.getDetailId();
        DataTypeEnum type = dto.getType();
        final FollowTypeEnum followTypeEnum = FollowTypeEnum.ofValue(type.getValue());
        final BizTypeEnum businessType = BizTypeEnum.ofValue(dto.getBusinessType().getValue());
        final String bizId = dto.getBizId();
        final Long userId = dto.getUserId();
        final Long shopId = dto.getShopId();
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                        .eq(FollowTask::getType, followTypeEnum)
                        .eq(FollowTask::getBizType, businessType)
                        .eq(FollowTask::getDetailId, detailId)
                        .eq(StringUtils.isNotBlank(bizId), FollowTask::getBizId, bizId)
                , Boolean.FALSE);
        if (Objects.isNull(task)) {
            return;
        }
        boolean isSecondTask = Boolean.TRUE.equals(dto.getSecondTask());
        boolean sameFollow = task.getFollowUser().equals(userId) && task.getFollowShop().equals(shopId);
        if (emptyMobile && sameFollow && !isSecondTask) {
            return;
        }
        if (isSecondTask) {
            BV.notNull(dto.getDeadline(), () -> "二次分配任务截止日期不能为空");
            task.setState(TaskStateEnum.ONGOING);
            task.setDeadline(DateUtil.date2LocalDateTime(dto.getDeadline()));
        }
        if (!sameFollow) {
            task.setFollowUser(userId);
            task.setFollowShop(shopId);
            TaskChangeLog changeLog = new TaskChangeLog();
            changeLog.setTaskId(task.getId());
            changeLog.setUserId(userId);
            changeLog.setShopId(shopId);
            changeLog.setPreUserId(task.getFollowUser());
            changeLog.setPreShopId(task.getFollowShop());
            taskChangeLogService.save(changeLog);

            LocalDate nextTime = Optional.ofNullable(dto.getRecommendNextTime()).map(DateUtil::date2LocalDate).orElse(LocalDate.now().plusDays(1L));
            taskNextPlanService.recordNextTime(task.getId(), nextTime);
        }
        if (!emptyMobile) {
            task.setContacts(dto.getMobile());
            List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(FollowRecord::getTaskId, task.getId())
                    .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                    .eq(FollowRecord::getYn, Boolean.TRUE)
            );
            if (!CollectionUtils.isEmpty(list)) {
                list.forEach(record -> record.setContacts(dto.getMobile()));
                followRecordService.updateBatchById(list);
            }
        }
        followTaskService.updateById(task);
        if (isSecondTask) {
            FollowStrategy strategy = followMap.get(followTypeEnum);
            strategy.secondTaskStopRecord(task.getId());
        }
    }

    /**
     * 主动战败
     *
     * @param currentUser
     * @param reason
     * @param recordId
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'defeat:' + #recordId", message = "请勿重复提交")
    public void defeat(LoginAuthBean currentUser, Long recordId, String reason) {
        FollowRecord record = followRecordService.getById(recordId);
        Long userId = currentUser.getUserId();
        BV.notNull(record, () -> "跟进信息不存在");
        BV.isTrue(record.getYn(), () -> "跟进信息不存在");
        boolean isSamePerson = userId.equals(record.getUserId());
        BV.isTrue(isSamePerson, () -> "非本人跟进，无法操作");
        // 记录主动放弃
        ApproveRecord approveRecord = new ApproveRecord();
        approveRecord.setDataId(record.getTaskId());
        approveRecord.setUserId(userId);
        approveRecord.setOrderNo(String.format("follow_defeat_%s", record.getId()));
        approveRecord.setReason(reason);
        approveRecord.setState(ApproveStateEnum.DONE);
        approveRecord.setType(ApproveTypeEnum.FOLLOW_DEFEAT);
        approveRecord.setPassed(Boolean.TRUE);
        approveRecordService.save(approveRecord);
        // 无需审批，默认审批通过
        onApproveAgree(record.getTaskId());
    }

    /**
     * 主动战败
     *
     * @param currentUser
     * @param reason
     * @param recordId
     * @deprecated 2023-08-17 废弃，当前需求不需要审批
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'defeat:' + #recordId", message = "请勿重复提交")
    public void defeatBak(LoginAuthBean currentUser, Long recordId, String reason) {
        FollowRecord record = followRecordService.getById(recordId);
        BV.notNull(record, () -> "跟进信息不存在");
        BV.isTrue(record.getYn(), () -> "跟进信息不存在");
        String flowNo = getPreFlowNo();
        Long userId = currentUser.getUserId();
        Long shopId = record.getShopId();
        if (BizTypeEnum.AS.equals(record.getBizType())) {
            flowNo = getAfterFlowNo();
        }
        boolean repetition = isRepetition(record.getTaskId().toString());
        BV.isFalse(repetition, () -> "正在审批处理中，请勿重复申请");

        FlowDto flowDto = FlowDto.create(currentUser.getGroupId(), flowNo, userId, shopId);
        HashMap<String, Object> param = new HashMap<>(2);
        param.put("recordId", recordId.toString());
        param.put("type", record.getType().getValue().toString());
        flowDto.setExtData(param);
        List<String> list = new ArrayList<>();
        list.add("申请人: " + currentUser.getUserName());
        if (BizTypeEnum.AS.equals(record.getBizType())) {
            list.add("客户: " + (StringUtils.isBlank(record.getCustomerName()) ? "-" : record.getCustomerName()));
            list.add("车辆: " + record.getPlateNo());
        }
        list.add("跟进类型: " + record.getType().getName());
        list.add("战败原因: " + reason);
        flowDto.setBriefContent(list);
        String orderNo = flowApproveRpc.initiate(flowDto);
        ApproveRecord approveRecord = new ApproveRecord();
        approveRecord.setDataId(record.getTaskId());
        approveRecord.setUserId(userId);
        approveRecord.setOrderNo(orderNo);
        approveRecord.setReason(reason);
        approveRecord.setState(ApproveStateEnum.WAIT);
        approveRecord.setType(ApproveTypeEnum.FOLLOW_DEFEAT);
        approveRecord.setPassed(Boolean.FALSE);
        approveRecordService.save(approveRecord);
    }

    /**
     * 审批战败同意
     *
     * @param taskId
     */
    @Transactional(rollbackFor = Exception.class)
    public void onApproveAgree(Long taskId) {
        FollowTask task = followTaskService.getById(taskId);
        if (Objects.isNull(task) || !TaskStateEnum.ONGOING.equals(task.getState())) {
            return;
        }
        task.setCloseTime(LocalDateTime.now());
        FollowStrategy strategy = followMap.get(task.getType());
        strategy.endTask(task, true);
        FollowRecord record = followRecordService.getOngoingRecord(taskId);
        if (Objects.isNull(record)) {
            return;
        }
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        record.setCloseTime(LocalDateTime.now());
        record.setReason(FollowCloseReason.ABANDON);
        strategy.markSecretReportHistory(record);
        followRecordService.updateById(record);
        BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), record.getId().toString(), new Date(), record.getShopId());
        todoRpcService.cancel(dto);

        FollowApproveDTO approveDTO = new FollowApproveDTO();
        approveDTO.setVin(task.getVin());
        approveDTO.setDetailId(task.getDetailId());
        approveDTO.setPlateNo(task.getPlateNo());
        approveDTO.setType(task.getType().getValue());
        approveDTO.setBusinessType(task.getBizType().getValue());
        approveDefeatFollowProducer.send(approveDTO);
    }

    /**
     * 跟进历史记录
     *
     * @param groupId
     * @param customerId
     * @param bizType
     * @return
     */
    public List<ClueHistoryVO> taskHistoryList(final Long groupId, final Long customerId, Integer bizType) {
        BizTypeEnum businessTypeEnum = BizTypeEnum.ofValue(bizType);
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getCustomerId, customerId)
                .eq(FollowTask::getGroupId, groupId)
                .ne(FollowTask::getType, FollowTypeEnum.AC)
                .eq(Objects.nonNull(businessTypeEnum), FollowTask::getBizType, businessTypeEnum));
        return getHistoryByTaskList(list);
    }

    /**
     * 售后跟进历史记录
     *
     * @param groupId
     * @param vin
     * @param type
     * @return
     */
    public List<ClueHistoryVO> asTaskHistoryList(final Long groupId, final String vin, FollowTypeEnum type) {
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getVin, vin)
                .eq(Objects.nonNull(type), FollowTask::getType, type)
                .eq(FollowTask::getGroupId, groupId)
                .eq(FollowTask::getBizType, BizTypeEnum.AS));

        return getHistoryByTaskList(list);
    }

    /**
     * 根据线索id查询跟进记录
     *
     * @param taskId
     * @return
     */
    public List<FollowRecordHistoryVO> queryHistoryByTaskId(Long taskId) {
        FollowTask task = followTaskService.getById(taskId);
        BV.notNull(task, () -> "跟进任务不存在");
        FollowStrategy strategy = followMap.get(task.getType());
        BV.notNull(strategy, () -> "跟进类型不正确");
        return strategy.queryHistoryByTask(task);
    }

    /**
     * 查询客户的跟进历史记录
     *
     * @param customerId 客户档案用户id
     * @param bizType    业态类型
     * @param type       跟进类型
     */
    public List<FollowRecordHistoryVO> queryHistoryByCustomer(Long customerId, Integer bizType, Integer type) {
        FollowTypeEnum typeEnum = FollowTypeEnum.ofValue(type);
        List<FollowRecord> followRecordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getCustomerId, customerId)
                .eq(FollowRecord::getBizType, bizType)
                .eq(FollowRecord::getFollowStatus, FollowStatusEnum.CLOSED)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(Objects.nonNull(typeEnum), FollowRecord::getType, typeEnum)
        );
        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        return transfromData(followRecordList);
    }

    /**
     * 查询车辆的跟进历史记录
     *
     * @param vin     vin
     * @param groupId 集团id
     */
    public List<FollowRecordHistoryVO> queryHistoryByVin(String vin, Long groupId) {
        List<FollowRecord> followRecordList = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getVin, vin)
                .eq(FollowRecord::getGroupId, groupId)
                .eq(FollowRecord::getFollowStatus, FollowStatusEnum.CLOSED)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        return transfromData(followRecordList);
    }

    /**
     * 查询跟进池详情
     *
     * @param taskId
     * @return
     */
    public FollowDetailVO followPoolDetail(Long taskId) {
        FollowTask task = followTaskService.getById(taskId);
        BV.notNull(task, () -> "跟进线索任务不存在，请刷新列表");
        FollowStrategy strategy = followMap.get(task.getType());
        Assert.notNull(strategy, "strategy cannot be null");
        return strategy.followPoolDetail(task);
    }

    /**
     * 根据referId查询跟进任务Id
     *
     * @param refererId
     * @param groupId
     * @param type
     * @return
     */
    public Long queryTaskByReferId(String refererId, Long groupId, Integer type) {
        FollowTypeEnum typeEnum = FollowTypeEnum.ofValue(type);
        BV.notNull(typeEnum, () -> "跟进类型不正确");
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getDetailId, refererId)
                .eq(FollowTask::getType, typeEnum)
                .eq(FollowTask::getGroupId, groupId)
                .last(" limit 1 ")
        );
        BV.notNull(task, () -> "数据异常，请刷新页面");
        return task.getId();
    }

    /**
     * 通过智能电话拨号
     *
     * @param recordId
     * @return
     */
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'call:' + #recordId", message = "请勿重复尝试")
    public String callSomeone(Long recordId) {
        final FollowRecord record = followRecordService.getById(recordId);
        BV.notNull(record, () -> "跟进信息不存在");
        BV.isTrue(record.getYn(), () -> "跟进信息不存在");
        Long memberId = record.getMemberId();
        String contacts = record.getContacts();
        if (Objects.nonNull(memberId) && memberId > 0) {
            MemberUserDTO memberUserDTO = memberRpcService.user(memberId);
            if (Objects.nonNull(memberUserDTO)) {
                String phone = memberUserDTO.getPhone();
                if (phone != null && !phone.isEmpty() && !phone.equals("#")) {
                    contacts = phone;
                    record.setContacts(phone);
                    followRecordService.updateById(record);
                }
            }
        }
        return Optional.ofNullable(contacts).orElse("#");
    }

    /**
     * 处理通话记录
     *
     * @param reportDTO
     */
    @Transactional(rollbackFor = Exception.class)
    public void readCallReport2Record(CallReportDTO reportDTO) {
        List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getUserId, reportDTO.getStaffId())
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .eq(FollowRecord::getContacts, reportDTO.getPeerNo())
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        boolean effectiveDial = reportDTO.getTalkTime() != null && reportDTO.getTalkTime() >= 30L;
        List<SecretReportHistory> reportHistoryList = new ArrayList<>();
        List<Long> ignoreTask = new ArrayList<>();
        for (FollowRecord record : list) {
            FollowAttachmentDTO dto = new FollowAttachmentDTO();
            FollowStrategy strategy = followMap.get(record.getType());
            dto.setTaskId(record.getTaskId());
            dto.setRecordId(record.getId());
            dto.setFeedbackType(FollowTypeEnum.AC.equals(record.getType()) ? FeedbackTypeEnum.OTHER.getValue() : null);
            dto.setAttType(AttTypeEnum.SMART_PHONE.getValue());
            dto.setAttachments(reportDTO.getCallId());
            if (effectiveDial) {
                record.setCalled(CallStateEnum.EFFECTIVE_DIAL);
                dto.setAccComplete(true);
                dto.setMark(Boolean.TRUE);
            }
            CallStateEnum called = record.getCalled();
            if (!CallStateEnum.EFFECTIVE_DIAL.equals(called)) {
                record.setCalled(CallStateEnum.INEFFECTIVE_DIAL);
            }
            SecretReportHistory reportHistory = createReportHistory(reportDTO, record);
            reportHistoryList.add(reportHistory);
            strategy.uploadAtt(dto, record, record.getUserId());
            ignoreTask.add(record.getTaskId());
        }
        reportDTO.setIgnoreTaskId(ignoreTask);
        followRecordService.updateBatchById(list);
        secretReportHistoryService.saveBatch(reportHistoryList);
    }

    /**
     * 处理通话记录
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public void readCallReport2Task(CallReportDTO dto) {
        if (Objects.isNull(dto.getMemberId()) || dto.getMemberId() <= 0) {
            MemberUserDTO memberInfo = memberRpcService.queryByMobile(dto.getPeerNo());
            if (Objects.nonNull(memberInfo)) {
                dto.setMemberId(memberInfo.getUserId());
                dto.setMemberName(memberInfo.getRealName());
            }
        }

        if (Objects.isNull(dto.getMemberId()) || dto.getMemberId() < 0) {
            return;
        }
        List<Long> ignoreTaskId = dto.getIgnoreTaskId();
        List<FollowTask> taskList = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getFollowUser, dto.getStaffId())
                .eq(FollowTask::getMemberId, dto.getMemberId())
                .eq(FollowTask::getGroupId, dto.getGroupId())
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .notIn(!CollectionUtils.isEmpty(ignoreTaskId), FollowTask::getId, ignoreTaskId));

        if (CollectionUtils.isEmpty(taskList)) {
            return;
        }
        for (FollowTask task : taskList) {
            uploadAttBySmartPhone(dto, task);
        }
    }

    private void uploadAttBySmartPhone(CallReportDTO reportDTO, FollowTask task) {
        FollowTypeEnum followType = task.getType();
        List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, task.getId())
                .eq(FollowRecord::getUserId, task.getFollowUser())
                .eq(FollowRecord::getType, followType)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                .eq(FollowRecord::getYn, Boolean.TRUE));
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        FollowStrategy strategy = followMap.get(followType);
        boolean effectiveDial = reportDTO.getTalkTime() != null && reportDTO.getTalkTime() >= 30L;
        List<SecretReportHistory> reportHistoryList = new ArrayList<>();
        for (FollowRecord record : list) {
            FollowAttachmentDTO dto = new FollowAttachmentDTO();
            dto.setTaskId(record.getTaskId());
            dto.setRecordId(record.getId());
            dto.setAttType(AttTypeEnum.SMART_PHONE.getValue());
            dto.setAttachments(reportDTO.getCallId());
            dto.setFeedbackType(FollowTypeEnum.AC.equals(record.getType()) ? FeedbackTypeEnum.OTHER.getValue() : null);
            if (effectiveDial) {
                record.setCalled(CallStateEnum.EFFECTIVE_DIAL);
                dto.setMark(Boolean.TRUE);
            }
            CallStateEnum called = record.getCalled();
            if (!CallStateEnum.EFFECTIVE_DIAL.equals(called)) {
                record.setCalled(CallStateEnum.EFFECTIVE_DIAL);
            }
            reportHistoryList.add(createReportHistory(reportDTO, record));
            strategy.uploadAtt(dto, record, record.getUserId());
        }
        followRecordService.updateBatchById(list);
        secretReportHistoryService.saveBatch(reportHistoryList);
    }


    /**
     * 审批防重
     *
     * @param taskId
     * @return
     */
    private boolean isRepetition(String taskId) {
        return approveRecordService.count(Wrappers.<ApproveRecord>lambdaQuery()
                .eq(ApproveRecord::getDataId, taskId)
                .eq(ApproveRecord::getType, ApproveTypeEnum.FOLLOW_DEFEAT)
                .eq(ApproveRecord::getState, ApproveStateEnum.WAIT)) > 0;
    }

    private SecretReportHistory createReportHistory(CallReportDTO reportDTO, FollowRecord record) {
        SecretReportHistory history = new SecretReportHistory();
        history.setTaskId(record.getTaskId());
        history.setTaskType(record.getType());
        history.setFollowRecordId(record.getId());
        history.setCallId(reportDTO.getCallId());
        history.setStaffId(reportDTO.getStaffId());
        history.setStaffName(reportDTO.getStaffName());
        history.setStaffMobile(reportDTO.getStaffMobile());
        history.setCustomerId(record.getCustomerId());
        history.setCustomerName(reportDTO.getMemberName());
        history.setCustomerMobile(reportDTO.getPeerNo());
        history.setCallType(reportDTO.getDialType());
        history.setCallTime(reportDTO.getCallTime());
        history.setCallDuration(reportDTO.getTalkTime());
        history.setShopId(record.getShopId());
        history.setGroupId(record.getGroupId());

        long count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                .eq(FollowRecordLog::getTaskId, record.getTaskId())
                .eq(FollowRecordLog::getAttType, AttTypeEnum.SMART_PHONE));
        history.setFirstCall(count <= 0);
        return history;
    }

    /**
     * 根据业务类型 档案id获取已完成的记录
     *
     * @param bizType
     * @param customerIdList
     * @return
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    public List<SucessFollowRecordVo> getCompletedFollowRecord(Integer bizType, List<Long> customerIdList) {
        List<SucessFollowRecord> list = followRecordService.getCompletedFollowRecord(bizType, customerIdList);
        if (null != list && !list.isEmpty()) {
            List<SucessFollowRecordVo> returnList = new ArrayList<>();

            for (SucessFollowRecord sucessFollowRecord : list) {
                SucessFollowRecordVo sucessFollowRecordVo = new SucessFollowRecordVo();
                BeanUtils.copyProperties(sucessFollowRecord, sucessFollowRecordVo);
                String activeName = null;
                if (StringUtils.isNotBlank(sucessFollowRecord.getNote())) {
                    JSONObject noteJson = JSON.parseObject(sucessFollowRecord.getNote());
                    if (null != noteJson && noteJson.containsKey("activityName")) {
                        activeName = noteJson.getString("activityName");
                    }
                }
                String followTypeName = FollowTypeEnum.getNameByVale(Integer.parseInt(sucessFollowRecord.getFollowType()));
                if (StringUtils.isBlank(activeName)) {
                    followTypeName += "跟进";
                } else {
                    followTypeName += "-" + activeName;
                }
                sucessFollowRecordVo.setFollowTypeName(followTypeName);
                returnList.add(sucessFollowRecordVo);
            }
            return returnList;
        }
        return null;
    }

    /**
     * 查询跟进记录池详情
     *
     * @param id
     * @return
     */
    public List<FollowRecordHistoryVO> recordPoolDetail(Long id) {
        FollowRecord record = followRecordService.getById(id);
        BV.notNull(record, () -> "跟进待办不存在");
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(Collections.singletonList(id));
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        MemberUserDTO user = memberRpcService.user(record.getMemberId());
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecordHistoryVO vo = getFollowRecordHistoryVO(attachment, Optional.ofNullable(user).map(MemberUserDTO::getRealName).orElse(null), record);
            list.add(vo);
        }

        return list;
    }

    /**
     * 根据跟进id查询其他正在进行中的线索任务
     *
     * @param recordId 跟进id
     * @return
     */
    public List<FollowTaskSimpleVO> queryUserOtherTask(final Long recordId) {
        FollowRecord record = followRecordService.getById(recordId);
        boolean efficient = Optional.ofNullable(record).map(r -> Boolean.TRUE.equals(r.getYn())).orElse(Boolean.FALSE);
        BV.isTrue(efficient, () -> "跟进任务不存在");
        final Long userId = record.getUserId();
        final Long memberId = record.getMemberId();
        if (BizTypeEnum.PS.equals(record.getBizType()) && (Objects.isNull(memberId) || memberId <= 0)) {
            return Collections.emptyList();
        }
        final String vin = record.getVin();
        List<FollowTask> list = followTaskService.list(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getBizType, record.getBizType())
                .eq(FollowTask::getFollowUser, userId)
                .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                .eq(BizTypeEnum.PS.equals(record.getBizType()), FollowTask::getMemberId, memberId)
                .eq(BizTypeEnum.AS.equals(record.getBizType()), FollowTask::getVin, vin)
        );
        if (CollectionUtils.isEmpty(list) || Boolean.TRUE.equals(record.getIgnoreReport())) {
            return Collections.emptyList();
        }
        List<FollowTaskSimpleVO> taskSimpleVOList = new ArrayList<>();
        Set<FollowTypeEnum> typeSet = new HashSet<>(ListUtil.toList(FollowTypeEnum.AC, FollowTypeEnum.FA, FollowTypeEnum.AF));
        for (FollowTask task : list) {
            boolean before = LocalDate.now().isBefore(task.getDeadline().minusDays(1L).toLocalDate());
            FollowTaskSimpleVO vo = FollowTaskSimpleVO.with(task, !typeSet.contains(task.getType()) && before);
            OriginalData originalData = originalDataService.getOne(Wrappers.<OriginalData>lambdaQuery()
                            .eq(OriginalData::getBizId, task.getBizId())
                            .eq(OriginalData::getDetailId, task.getDetailId())
                            .eq(OriginalData::getType, task.getType())
                    , Boolean.FALSE);
            if (Objects.nonNull(originalData)) {
                JSONObject noteJson = JSON.parseObject(originalData.getNote());
                if (null != noteJson && noteJson.containsKey("activityName")) {
                    vo.setTaskName(noteJson.getString("activityName"));
                }
            }
            taskSimpleVOList.add(vo);
        }

        return taskSimpleVOList;
    }

    public List<ViewDataVO> queryFollowRatio(FollowRatioQuery query) {
        List<Long> shopIds = query.getShopIds();
        String bizId = query.getBizId();
        BizTypeEnum bizTypeEnum = BizTypeEnum.ofValue(query.getBizType());
        FollowTypeEnum followTypeEnum = FollowTypeEnum.ofValue(query.getFollowType());
        List<ViewDataVO> list = new ArrayList<>();
        int totalCount = followRecordService.count(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getBizId, bizId)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .in(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING, FollowStatusEnum.CLOSED)
                .eq(Objects.nonNull(followTypeEnum), FollowRecord::getType, followTypeEnum)
                .eq(Objects.nonNull(bizTypeEnum), FollowRecord::getBizType, bizTypeEnum)
                .in(!CollectionUtils.isEmpty(shopIds), FollowRecord::getShopId, shopIds)
        );
        if (totalCount == 0) {
            return list;
        }
        int alreadyCount = followRecordService.count(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getBizId, bizId)
                .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .eq(FollowRecord::getFollowStatus, FollowStatusEnum.CLOSED)
                .eq(FollowRecord::getReason, FollowCloseReason.DONE)
                .eq(Objects.nonNull(followTypeEnum), FollowRecord::getType, followTypeEnum)
                .eq(Objects.nonNull(bizTypeEnum), FollowRecord::getBizType, bizTypeEnum)
                .in(!CollectionUtils.isEmpty(shopIds), FollowRecord::getShopId, shopIds)
        );
        BigDecimal alreadyRation = BigDecimal.valueOf(alreadyCount).divide(BigDecimal.valueOf(totalCount), 4, RoundingMode.HALF_UP);
        list.add(new ViewDataVO(alreadyCount, "已跟进", alreadyRation));

        int onGoingCount = totalCount - alreadyCount;
        BigDecimal onGoingRation = BigDecimal.ONE.subtract(alreadyRation);
        list.add(new ViewDataVO(onGoingCount, "未跟进", onGoingRation));
        return list;
    }


    /**
     * 根据跟进记录id获取详情及附件信息
     *
     * @param recordId
     * @return
     */
    public List<DateFollowRecordHistoryVO> followRecordDetail(Long recordId) {
        FollowRecord record = followRecordService.getById(recordId);
        if (Objects.isNull(record) || Boolean.FALSE.equals(record.getYn())) {
            return null;
        }
        //查询附件
        List<Long> recordIdList = new ArrayList<>();
        recordIdList.add(recordId);
        List<FollowRecordLog> followRecordLogs = followRecordLogService.getListByRecordIds(recordIdList);
        if (null != followRecordLogs && !followRecordLogs.isEmpty()) {
            //按日期分组
            List<DateFollowRecordHistoryVO> dateGroupList = new ArrayList<>();
            Map<String, DateFollowRecordHistoryVO> tmpMap = new HashMap<>();

            for (FollowRecordLog attachment : followRecordLogs) {
                FollowRecordHistoryVO vo = new FollowRecordHistoryVO()
                        .setAttachments(attachment.getAttachments())
                        .setId(attachment.getId())
                        .setRecordId(attachment.getRecordId())
                        .setTaskId(record.getTaskId())
                        .setAttType(attachment.getAttType())
                        .setFeedbackType(attachment.getFeedbackType())
                        .setFollowType(record.getType())
                        .setUploadTime(attachment.getUploadTime())
                        .setDescribes(attachment.getDescribes())
                        .setUserName(record.getUserName());
                //获取年月日
                String ymd = DateUtil.getFormatString(DateUtil.toDate(attachment.getUploadTime()), "yyyy-MM-dd");
                DateFollowRecordHistoryVO curFollowRecord = null;
                if (tmpMap.containsKey(ymd)) {
                    curFollowRecord = tmpMap.get(ymd);
                } else {
                    curFollowRecord = new DateFollowRecordHistoryVO();
                    curFollowRecord.setYmd(attachment.getUploadTime().toLocalDate());
                    curFollowRecord.setFollowRecordLogList(new ArrayList<>());
                    dateGroupList.add(curFollowRecord);
                    tmpMap.put(ymd, curFollowRecord);
                }
                curFollowRecord.getFollowRecordLogList().add(vo);
            }
            return dateGroupList;
        }
        return new ArrayList<>();
    }

    /**
     * 查询未完成的待办数量
     *
     * @param userId
     * @param bizType
     * @return
     */
    public long queryOngoingRecord(Long userId, BizTypeEnum bizType) {
        return followRecordService.recordRemaining(userId, bizType, null);
    }

    private List<ClueHistoryVO> getHistoryByTaskList(List<FollowTask> list) {
        List<ClueHistoryVO> historyVOList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            historyVOList.addAll(list.stream().map(ClueHistoryVO::with).filter(Objects::nonNull).collect(Collectors.toList()));
        }

        for (ClueHistoryVO clueHistoryVO : historyVOList) {
            if (StringUtils.isNotBlank(clueHistoryVO.getBizId()) && FollowTypeEnum.AF.getValue().equals(clueHistoryVO.getType())) {
                OriginalData originalData = originalDataService.getOne(Wrappers.<OriginalData>lambdaQuery()
                                .eq(OriginalData::getBizId, clueHistoryVO.getBizId())
                                .eq(OriginalData::getDetailId, clueHistoryVO.getDetailId())
                                .eq(OriginalData::getType, clueHistoryVO.getType())
                        , Boolean.FALSE);
                if (Objects.nonNull(originalData)) {
                    JSONObject noteJson = JSON.parseObject(originalData.getNote());
                    if (null != noteJson && noteJson.containsKey("activityName")) {
                        clueHistoryVO.setActivityName(noteJson.getString("activityName"));
                    }
                }
            }

            if (StringUtils.isEmpty(clueHistoryVO.getActivityName())) {
                clueHistoryVO.setActivityName(clueHistoryVO.getTypeDesc());
            }

            if (StringUtils.isEmpty(clueHistoryVO.getUserName())) {
                String useName = Optional.ofNullable(ehrRpcService.user(clueHistoryVO.getUserId())).map(UserInfoDTO::getUserName).orElse(null);
                clueHistoryVO.setUserName(useName);
            }
            if (StringUtils.isEmpty(clueHistoryVO.getShopName())) {
                String shopName = Optional.ofNullable(oopService.shop(clueHistoryVO.getShopId())).map(ShopDTO::getShortName).orElse(null);
                clueHistoryVO.setShopName(shopName);
            }
        }
        historyVOList.sort(Comparator.comparing(ClueHistoryVO::getDeadline));
        return historyVOList;
    }

    private List<FollowRecordHistoryVO> transfromData(List<FollowRecord> followRecordList) {
        List<Long> recordIds = followRecordList.stream().map(FollowRecord::getId).collect(Collectors.toList());
        Map<Long, FollowRecord> map = followRecordList.stream().collect(Collectors.toMap(FollowRecord::getId, r -> r));
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(recordIds);
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecord followRecord = map.get(attachment.getRecordId());
            if (Objects.isNull(followRecord)) {
                continue;
            }
            FollowRecordHistoryVO vo = getFollowRecordHistoryVO(attachment, followRecord.getCustomerName(), followRecord);
            list.add(vo);
        }

        return list;
    }

    @NotNull
    private static FollowRecordHistoryVO getFollowRecordHistoryVO(FollowRecordLog attachment, String followRecord, FollowRecord followRecord1) {
        FollowRecordHistoryVO vo = new FollowRecordHistoryVO();
        vo.setAttachments(attachment.getAttachments());
        vo.setId(attachment.getId());
        vo.setCustomerName(followRecord);
        vo.setRecordId(attachment.getRecordId());
        vo.setTaskId(attachment.getTaskId());
        vo.setAttType(attachment.getAttType());
        vo.setFeedbackType(attachment.getFeedbackType());
        vo.setFollowType(followRecord1.getType());
        vo.setUploadTime(attachment.getUploadTime());
        vo.setDescribes(attachment.getDescribes());
        vo.setUserName(followRecord1.getUserName());
        return vo;
    }
}
