package cn.fw.shirasawa.service.bus.follow;

import cn.fw.shirasawa.common.constant.LuaScript;
import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.enums.FollowCloseReason;
import cn.fw.shirasawa.domain.enums.FollowStatusEnum;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.enums.TaskStateEnum;
import cn.fw.shirasawa.rpc.backlog.TodoRpcService;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.rpc.ehr.EhrRpcService;
import cn.fw.shirasawa.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.shirasawa.service.data.FollowRecordService;
import cn.fw.shirasawa.service.data.FollowTaskService;
import cn.hutool.core.collection.ListUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 跟进待办执行服务
 *
 * @author : kurisu
 * @version : 2.0
 * @className : FollowExecuteService
 * @description : 跟进待办执行服务
 * @date : 2023-04-25 17:36
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class FollowExecuteService {
    /**
     * 有最大限制的跟进类型
     */
    private static final Set<FollowTypeEnum> MAX_LIMIT_TYPE_SET = new HashSet<>(ListUtil.toList(FollowTypeEnum.AF, FollowTypeEnum.CF, FollowTypeEnum.PF, FollowTypeEnum.FM, FollowTypeEnum.RM, FollowTypeEnum.IR));

    private final FollowTaskService followTaskService;
    private final FollowRecordService followRecordService;
    private final EhrRpcService ehrRpcService;
    private final TodoRpcService todoRpcService;
    private final StringRedisTemplate stringRedisTemplate;


    @Value("${spring.cache.custom.global-prefix}:task")
    @Getter
    private String prefix;

    @Value("${follow.todo.single-type-max-limit}")
    @Getter
    private Integer maxLimit;

    @Transactional(rollbackFor = Exception.class)
    public void execute(List<FollowRecord> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<FollowRecord> needUpdateList = new ArrayList<>();
        List<Long> failIdList = new ArrayList<>();
        for (FollowRecord record : list) {
            Long userId = record.getUserId();
            StaffInfoDTO infoDTO = ehrRpcService.queryStaffInfo(userId);
            if (Objects.isNull(infoDTO) || !Integer.valueOf(2).equals(infoDTO.getStaffStatus())) {
                continue;
            }
            FollowTask task = followTaskService.getById(record.getTaskId());
            if (Objects.nonNull(task) && TaskStateEnum.ONGOING.equals(task.getState())) {
                boolean check = preCheck(record, task);
                if (check) {
                    Duration followDuration = Optional.ofNullable(task.getFollowDuration()).orElse(Duration.ofHours(36L));
                    execute(record, needUpdateList, followDuration);
                }
            } else {
                failIdList.add(record.getId());
            }
        }
        if (!CollectionUtils.isEmpty(needUpdateList)) {
            followRecordService.updateBatchById(needUpdateList);
        }
        if (!CollectionUtils.isEmpty(failIdList)) {
            followRecordService.voidByRecordIds(failIdList);
        }
    }

    /**
     * 执行推送待办
     *
     * @param record         待办
     * @param needUpdateList 需要更新的record 包含推送后该状态
     */
    private void execute(FollowRecord record, List<FollowRecord> needUpdateList, Duration drumTime) {
        if (!MAX_LIMIT_TYPE_SET.contains(record.getType())) {
            record.setTodoTime(LocalDateTime.now());
            if (!FollowTypeEnum.AC.equals(record.getType())) {
                record.setDeadline(LocalDateTime.now().plus(drumTime));
            }
            boolean execute = pushTodo(record);
            if (execute) {
                record.setAddTodo(Boolean.TRUE);
                needUpdateList.add(record);
            }
        } else {
            final String cacheName = getMaxLimitCacheKey(record.getType(), record.getUserId());
            Long count = exatAndIncrby(cacheName, DateUtil.toDate(LocalDate.now().plusDays(1L).atStartOfDay()));
            BV.notNull(count, () -> "获取当日推送的跟进数失败");
            if (maxLimit >= count) {
                record.setTodoTime(LocalDateTime.now());
                record.setDeadline(LocalDateTime.now().plus(drumTime));
                boolean execute = pushTodo(record);
                if (execute) {
                    record.setAddTodo(Boolean.TRUE);
                    needUpdateList.add(record);
                } else {
                    stringRedisTemplate.opsForValue().decrement(cacheName);
                }
            } else {
                record.setPlanTime(record.getPlanTime().plusDays(1L));
                record.setDeadline(record.getDeadline().plusDays(1L));
                needUpdateList.add(record);
            }
        }
    }

    /**
     * 自增并且设置过期时间
     *
     * @param key                key
     * @param expireMilliseconds 过期时长[毫秒]
     * @return
     */
    public Long exAndIncrby(final String key, final long expireMilliseconds) {
        return incrby(LuaScript.LUR_SCRIPT_INCR_EX, key, expireMilliseconds);
    }

    /**
     * 自增并且设置过期时间
     *
     * @param key        key
     * @param expireTime 过期时间
     * @return
     */
    public Long exatAndIncrby(final String key, final Date expireTime) {
        long increment = Optional.ofNullable(stringRedisTemplate.opsForValue().increment(key)).orElse(0L);
        if (increment <= 2) {
            stringRedisTemplate.expireAt(key, expireTime);
        }
        return increment;
    }

    public Long exatAndIncrbyLua(final String key, final Date expireTime) {
        return incrby(LuaScript.LUR_SCRIPT_INCR_EXAT, key, expireTime.getTime());
    }


    private Long incrby(final String luaScriptStr, final String key, final long et) {
        Object[] args = new Object[]{
                String.valueOf(1),
                String.valueOf(et)
        };
        return stringRedisTemplate.execute(new DefaultRedisScript<>(luaScriptStr, Long.class), Collections.singletonList(key), args);
    }

    private boolean preCheck(FollowRecord record, FollowTask task) {
        LocalDateTime taskDeadline = task.getDeadline();
        LocalDateTime planTime = record.getPlanTime();
        if (record.getDeadline().isAfter(taskDeadline)) {
            record.setDeadline(taskDeadline);
        }
        if (planTime.isBefore(record.getDeadline())) {
            return true;
        } else {
            record.setFollowStatus(FollowStatusEnum.CLOSED);
            record.setReason(FollowCloseReason.OVERDUE);
            record.setCloseTime(LocalDateTime.now());
            followRecordService.updateById(record);
            return false;
        }
    }

    private boolean pushTodo(FollowRecord record) {
        String note = record.getNote();
        String todoCode = record.getTodoCode();
        HashMap<String, String> dynamicMap = Optional.ofNullable(JSONObject.<HashMap<String, String>>parseObject(note, HashMap.class)).orElse(new HashMap<>(4));
        dynamicMap.put("type", record.getType().getValue().toString());
        dynamicMap.put("taskId", record.getTaskId().toString());
        dynamicMap.put("id", record.getId().toString());
        BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), todoCode, String.valueOf(record.getId()), DateUtil.toDate(record.getTodoTime()), record.getShopId());
        dto.setExpireTime(DateUtil.toDate(record.getDeadline()));
        dto.setDynamicMap(dynamicMap);
        return todoRpcService.push(dto);
    }


    private String getMaxLimitCacheKey(@NonNull FollowTypeEnum typeEnum, @NonNull Long userId) {
        String today = LocalDate.now().toString();
        return String.format("%s:max-record:%s:%s:user:%s", getPrefix(), today, typeEnum.name(), userId);
    }


}
