package cn.fw.shirasawa.service.bus.follow.strategy;

import cn.fw.common.util.ValidationUtils;
import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.common.utils.StringUtils;
import cn.fw.shirasawa.common.utils.ThreadPoolUtil;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowRecordLog;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.other.OriginalData;
import cn.fw.shirasawa.domain.db.other.SecretReportHistory;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.dto.FollowTaskCompleteDTO;
import cn.fw.shirasawa.domain.dto.FollowTerminationDTO;
import cn.fw.shirasawa.domain.dto.RecordCompleteDTO;
import cn.fw.shirasawa.domain.enums.*;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowRecordHistoryVO;
import cn.fw.shirasawa.rpc.backlog.TodoRpcService;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.rpc.ehr.EhrRpcService;
import cn.fw.shirasawa.rpc.erp.UserService;
import cn.fw.shirasawa.rpc.erp.dto.UserInfoDTO;
import cn.fw.shirasawa.rpc.member.MemberRpcService;
import cn.fw.shirasawa.rpc.member.dto.MemberUserDTO;
import cn.fw.shirasawa.rpc.oop.OopService;
import cn.fw.shirasawa.rpc.pstn.PstnRpcService;
import cn.fw.shirasawa.service.data.*;
import cn.fw.shirasawa.service.event.CancelApproveEvent;
import cn.fw.shirasawa.service.event.RecordCompleteEvent;
import cn.fw.shirasawa.service.property.FollowTodoProperty;
import cn.hutool.core.collection.ListUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : AbstractFollowStrategy
 * @description : 策略抽象类
 * @date: 2020-08-17 10:42
 */
@Slf4j
@SuppressWarnings("Duplicates")
public abstract class AbstractFollowStrategy implements FollowStrategy {
    @Autowired
    protected FollowTaskService followTaskService;
    @Autowired
    protected FollowRecordService followRecordService;
    @Autowired
    protected UserService userService;
    @Autowired
    protected FollowRecordLogService followRecordLogService;
    @Autowired
    protected OopService oopService;
    @Autowired
    protected ApplicationEventPublisher eventPublisher;
    @Autowired
    protected TodoRpcService todoRpcService;
    @Autowired
    protected StringRedisTemplate redisTemplate;
    @Autowired
    protected EhrRpcService ehrRpcService;
    @Autowired
    protected PstnRpcService pstnRpcService;
    @Autowired
    protected SecretReportHistoryService secretReportHistoryService;
    @Autowired
    protected MemberRpcService memberRpcService;
    @Autowired
    protected TaskNextPlanService taskNextPlanService;
    @Autowired
    protected FollowTodoProperty todoProperty;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean origin2task(OriginalData originalData) {
        if (!duplicateCheck(originalData)) {
            final FollowTask task = this.transformTask(originalData);
            if (StringUtils.isEmpty(task.getContacts())) {
                task.setContacts("#");
            }

            boolean isOld = false;
            if (Objects.nonNull(task.getFollowDuration())) {
                isOld = Duration.ZERO.compareTo(task.getFollowDuration()) >= 0;
                if (isOld) {
                    task.setState(TaskStateEnum.DEFEAT);
                }
            }
            boolean saved = followTaskService.save(task);
            BV.isTrue(saved, () -> "生成task失败");
            if (isOld) {
                return saved;
            }

            FollowRecord record = createRecord(task, r -> {
                r.setPlanTime(task.getBeginTime());
                Duration duration = Optional.ofNullable(task.getFollowDuration()).orElse(Duration.ofHours(36L));
                r.setDeadline(task.getBeginTime().plus(duration));
            });
            record.setNote(originalData.getNote());
            if (FollowTypeEnum.SF.equals(task.getType())) {
                record.setCompelTel(Boolean.FALSE);
            } else if (BizTypeEnum.PS.equals(task.getBizType())) {
                assembleCompleteTelStatus(record, task);
            } else {
                if (FollowTypeEnum.AF.equals(task.getType())) {
                    record.setCompelTel(Boolean.FALSE);
                } else {
                    record.setCompelTel(ValidationUtils.checkMobile(record.getContacts()) || (Objects.nonNull(record.getMemberId()) && record.getMemberId() > 0));
                }
            }
            if (StringUtils.isEmpty(record.getContacts())) {
                record.setContacts(originalData.getContacts());
            }
            Map<String, String> map = todoProperty.getCodeMap();
            String todoCode = map.get(record.getType().name());
            record.setTodoCode(todoCode);
            if (FollowTypeEnum.AF.equals(task.getType())) {
                String[] split = todoCode.split(",");
                String code = BizTypeEnum.PS.equals(record.getBizType()) ? split[0] : split[1];
                record.setTodoCode(code);
            }
            return checkAndOverride(record);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void overdue(FollowRecord record) {
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        record.setCloseTime(record.getDeadline().minusSeconds(1L));
        record.setReason(FollowCloseReason.OVERDUE);
        markSecretReportHistory(record);
        followRecordService.updateById(record);
        eventPublisher.publishEvent(new RecordCompleteEvent(record, null));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void complete(FollowRecord record, Long userId, LocalDate nextDate) {
        RecordCompleteDTO.NextDTO nextDTO = new RecordCompleteDTO.NextDTO();
        nextDTO.setTaskId(record.getTaskId());
        nextDTO.setNextDate(nextDate);
        complete(record, userId, Collections.singletonList(nextDTO));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void complete(FollowRecord record, Long userId, List<RecordCompleteDTO.NextDTO> nextDto) {
        FollowTask task = followTaskService.getById(record.getTaskId());
        BV.isTrue(record.getUserId().equals(userId), () -> "无法完成非本人待办任务");

        if (FollowStatusEnum.CLOSED.equals(record.getFollowStatus()) && FollowCloseReason.DONE.equals(record.getReason())) {
            BackLogItemDTO backLogItemDTO = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), String.valueOf(record.getId()),
                    DateUtil.toDate(record.getCloseTime()), record.getShopId());
            todoRpcService.complete(backLogItemDTO);
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        if (!FollowStatusEnum.ONGOING.equals(record.getFollowStatus())) {
            return;
        }
        checkTelFollowStatus(record);
        record.setCloseTime(now);
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        record.setReason(FollowCloseReason.DONE);
        markSecretReportHistory(record);
        followRecordService.updateById(record);

        if (Objects.nonNull(task)) {
            int times = task.getTimes() == null ? 0 : task.getTimes();
            task.setTimes(times + 1);
            followTaskService.updateById(task);
        }

        if (!CollectionUtils.isEmpty(nextDto)) {
            for (RecordCompleteDTO.NextDTO next : nextDto) {
                if (FollowTypeEnum.AC.equals(record.getType()) && record.getTaskId().equals(next.getTaskId())) {
                    continue;
                }
                taskNextPlanService.recordNextTime(next.getTaskId(), next.getNextDate());
            }
        }

        CompletableFuture.runAsync(() -> {
            BackLogItemDTO backLogItemDTO = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), String.valueOf(record.getId()),
                    DateUtil.toDate(record.getCloseTime()), record.getShopId());
            todoRpcService.complete(backLogItemDTO);
            eventPublisher.publishEvent(new RecordCompleteEvent(record, task));
        }, ThreadPoolUtil.getInstance().getExecutor());
    }

    /**
     * @param task
     * @param nextTime
     * @param nextEndTime
     * @param note
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createNewRecord(FollowTask task, LocalDateTime nextTime, LocalDateTime nextEndTime, String note) {
        if (Objects.isNull(nextTime)) {
            return;
        }
        FollowRecord record = this.createRecord(task, nextTime, nextTime.minusHours(36L));
        updateContacts(record);
        record.setNote(note);

        checkAndOverride(record);
    }

    /**
     * 生成新待办
     *
     * @param task
     * @param nextTime
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void genNextRecord(FollowTask task, LocalDate nextTime) {
        if (Objects.isNull(nextTime)) {
            return;
        }
        final Duration duration = Optional.ofNullable(task.getFollowDuration()).orElse(Duration.ofHours(36L));
        FollowRecord record = createRecord(task, r -> {
            LocalDateTime nextDateTime = nextTime.atStartOfDay();
            r.setPlanTime(nextDateTime);
            r.setDeadline(nextDateTime.plus(duration));
        });
        FollowRecord recordOne = followRecordService.getOne(Wrappers.<FollowRecord>lambdaQuery()
                .eq(FollowRecord::getTaskId, task.getId())
                .eq(FollowRecord::getYn, Boolean.TRUE)
                .orderByDesc(FollowRecord::getId)
                .last(" limit 1")
        );
        if (Objects.isNull(recordOne)) {
            return;
        }
        record.setNote(recordOne.getNote());
        if (StringUtils.isEmpty(record.getContacts())) {
            record.setContacts(recordOne.getContacts());
        }
        assembleCompleteTelStatus(record, task);
        Map<String, String> map = todoProperty.getCodeMap();
        String todoCode = map.get(record.getType().name());
        record.setTodoCode(todoCode);
        if (FollowTypeEnum.AF.equals(task.getType())) {
            String[] split = todoCode.split(",");
            String code = BizTypeEnum.PS.equals(record.getBizType()) ? split[0] : split[1];
            record.setTodoCode(code);
        }
        checkAndOverride(record);
        taskNextPlanService.recordNextTime(task.getId(), task.getDeadline().plusDays(5L).toLocalDate());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void completeTask(FollowTaskCompleteDTO taskCompleteDTO) {
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                        .eq(FollowTask::getDetailId, taskCompleteDTO.getDetailId())
                        .eq(FollowTask::getType, taskCompleteDTO.getFollowType())
                        .eq(FollowTask::getState, TaskStateEnum.ONGOING)
                        .eq(FollowTask::getBizType, taskCompleteDTO.getBizType())
                , Boolean.FALSE
        );
        if (Objects.isNull(task)) {
            return;
        }
        task.setState(TaskStateEnum.COMPLETE);
        task.setCloseTime(DateUtil.date2LocalDateTime(taskCompleteDTO.getCompleteTime()));
        followTaskService.updateById(task);
        taskNextPlanService.stopFollow(task.getId());
        FollowRecord record = followRecordService.getOne(Wrappers.<FollowRecord>lambdaQuery()
                        .eq(FollowRecord::getTaskId, task.getId())
                        .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                , Boolean.FALSE
        );
        if (Objects.isNull(record)) {
            return;
        }
        record.setCloseTime(task.getCloseTime());
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        record.setReason(FollowCloseReason.EARLY);
        if (!Boolean.TRUE.equals(record.getAddTodo())) {
            record.setAddTodo(Boolean.TRUE);
            record.setTodoTime(LocalDateTime.now());
        }
        markSecretReportHistory(record);
        followRecordService.updateById(record);

        if (Boolean.TRUE.equals(record.getAddTodo())) {
            BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), record.getId().toString());
            todoRpcService.cancel(dto);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void terminationTaskPlan(FollowTerminationDTO terminationDTO) {
        FollowTask task = followTaskService.getOne(Wrappers.<FollowTask>lambdaQuery()
                        .eq(FollowTask::getDetailId, terminationDTO.getDetailId())
                        .eq(FollowTask::getType, terminationDTO.getFollowType())
                        .in(FollowTask::getState, TaskStateEnum.ONGOING)
                        .eq(FollowTask::getBizType, terminationDTO.getBizType())
                , Boolean.FALSE
        );
        if (Objects.isNull(task)) {
            return;
        }
        FollowCloseReason reason = terminationDTO.getReason();
        task.setCloseTime(LocalDateTime.now());
        task.setState(TaskStateEnum.DEFEAT);
        task.setInitiative(TaskDefeatTypeEnum.ofValue(reason.getValue()));

        followTaskService.updateById(task);
        taskNextPlanService.stopFollow(task.getId());

        FollowRecord record = followRecordService.getOngoingRecord(task.getId());
        if (Objects.isNull(record)) {
            return;
        }
        record.setReason(reason);
        if (FollowCloseReason.ROLE_CHANGE.equals(reason)) {
            record.setReason(FollowCloseReason.ROLE_CHANGE);
            record.setRemark(record.getRemark() + "+角色调整");
        }
        record.setCloseTime(LocalDateTime.now());
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        if (!Boolean.TRUE.equals(record.getAddTodo())) {
            record.setAddTodo(Boolean.TRUE);
            record.setTodoTime(LocalDateTime.now());
        }
        markSecretReportHistory(record);
        followRecordService.updateById(record);
        if (Boolean.TRUE.equals(record.getAddTodo())) {
            BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), record.getId().toString());
            todoRpcService.cancel(dto);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void endTask(FollowTask task, boolean fromFlow) {
        if (TaskStateEnum.ONGOING == task.getState()) {
            task.setState(TaskStateEnum.DEFEAT);
            task.setInitiative(TaskDefeatTypeEnum.A);
            if (fromFlow) {
                task.setInitiative(TaskDefeatTypeEnum.B);
            }
            if (Objects.isNull(task.getCloseTime())) {
                task.setCloseTime(task.getDeadline().minusSeconds(1L));
            }
            followTaskService.updateById(task);
            taskNextPlanService.stopFollow(task.getId());
        }
        if (!fromFlow) {
            // 任务到期的场景
            FollowRecord record = followRecordService.getOngoingRecord(task.getId());
            if (Objects.nonNull(record)) {
                record.setCloseTime(task.getCloseTime());
                record.setFollowStatus(FollowStatusEnum.CLOSED);
                record.setReason(FollowCloseReason.LEAD_EXPIRES);
                followRecordService.updateById(record);
                BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), record.getId().toString());
                todoRpcService.cancel(dto);
            }
            CancelApproveEvent event = new CancelApproveEvent(task.getId(), ApproveTypeEnum.FOLLOW_DEFEAT);
            eventPublisher.publishEvent(event);
        }
    }

    @Override
    public FollowDetailVO queryDetail(FollowRecord record) {
        FollowTask followTask = followTaskService.getById(record.getTaskId());
        BV.notNull(followTask, "跟进任务不存在");
        FollowDetailVO vo = new FollowDetailVO();
        assembleRecordDetail(vo, record, followTask);
        vo.setTaskDeadline(followTask.getDeadline());
        return vo;
    }

    @Override
    public List<FollowRecordHistoryVO> queryHistoryByRecord(FollowRecord record) {
        FollowTask task = followTaskService.getById(record.getTaskId());
        List<FollowRecordHistoryVO> list = new ArrayList<>(queryHistoryByTask(task));
        return list.stream().sorted((a, b) -> b.getId().compareTo(a.getId())).collect(Collectors.toList());
    }

    @Override
    public List<FollowRecordHistoryVO> queryHistoryByTask(FollowTask task) {
        List<FollowRecord> followRecordList = new ArrayList<>();
        List<FollowRecord> recordList = followRecordService.getRecordList(task.getId());
        if (!CollectionUtils.isEmpty(recordList)) {
            followRecordList.addAll(recordList);
        }
        if (CollectionUtils.isEmpty(followRecordList)) {
            return new ArrayList<>();
        }
        List<Long> recordIds = followRecordList.stream().map(FollowRecord::getId).collect(Collectors.toList());
        Map<Long, String> map = followRecordList.stream().collect(Collectors.toMap(FollowRecord::getId, r -> Optional.ofNullable(r.getUserName()).orElse("")));
        List<FollowRecordLog> attachments = followRecordLogService.getListByRecordIds(recordIds);
        if (CollectionUtils.isEmpty(attachments)) {
            return new ArrayList<>();
        }
        MemberUserDTO user = memberRpcService.user(task.getMemberId());
        List<FollowRecordHistoryVO> list = new ArrayList<>();
        for (FollowRecordLog attachment : attachments) {
            FollowRecordHistoryVO vo = new FollowRecordHistoryVO();
            vo.setAttachments(attachment.getAttachments());
            vo.setId(attachment.getId());
            vo.setCustomerName(Optional.ofNullable(user).map(MemberUserDTO::getRealName).orElse(null));
            vo.setRecordId(attachment.getRecordId());
            vo.setTaskId(attachment.getTaskId());
            vo.setAttType(attachment.getAttType());
            vo.setFeedbackType(attachment.getFeedbackType());
            vo.setFollowType(task.getType());
            vo.setUploadTime(attachment.getUploadTime());
            vo.setDescribes(attachment.getDescribes());
            vo.setUserName(map.get(attachment.getRecordId()));
            list.add(vo);
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadAtt(FollowAttachmentDTO dto, FollowRecord record, Long userId) {
        BV.isTrue(record.getUserId().equals(userId), () -> "无法跟进非本人待办任务");
        BV.isTrue(FollowStatusEnum.ONGOING.equals(record.getFollowStatus()), () -> "无法跟进已关闭待办任务");

        if (Objects.isNull(dto.getId())) {
            FollowRecordLog followRecordLog = followRecordLogService.getOne(Wrappers.<FollowRecordLog>lambdaQuery()
                            .eq(FollowRecordLog::getRecordId, record.getId())
                            .eq(FollowRecordLog::getTaskId, record.getTaskId())
                            .eq(FollowRecordLog::getAttType, dto.getAttType())
                            .eq(FollowRecordLog::getAttachments, dto.getAttachments())
                    , Boolean.FALSE
            );
            if (Objects.nonNull(followRecordLog)) {
                return;
            }
            AttTypeEnum attTypeEnum = dto.getAttTypeEnum();

            FollowRecordLog recordLog = new FollowRecordLog();
            recordLog.setId(dto.getId());
            recordLog.setTaskId(record.getTaskId());
            recordLog.setAttachments(dto.getAttachments());
            recordLog.setFeedbackType(dto.getFeedbackTypeEnum());
            recordLog.setAttType(attTypeEnum);
            recordLog.setDescribes(dto.getDescribes());
            recordLog.setRecordId(dto.getRecordId());
            if (Objects.isNull(dto.getId())) {
                recordLog.setUploadTime(LocalDateTime.now());
            }
            if (AttTypeEnum.SMART_PHONE.equals(attTypeEnum)) {
                recordLog.setMark(Boolean.TRUE.equals(dto.getMark()));
            } else {
                recordLog.setMark(Boolean.TRUE);
            }
            followRecordLogService.saveOrUpdate(recordLog);
            feedbackTask(record.getTaskId(), dto.getFeedbackTypeEnum());
            return;
        }

        FollowRecordLog followRecordLog = followRecordLogService.getById(dto.getId());
        if (Objects.nonNull(followRecordLog)) {
            if (Objects.isNull(followRecordLog.getFeedbackType())) {
                followRecordLog.setFeedbackType(dto.getFeedbackTypeEnum());
                followRecordLog.setDescribes(dto.getDescribes());
                followRecordLogService.updateById(followRecordLog);
                feedbackTask(record.getTaskId(), dto.getFeedbackTypeEnum());
            }
        }
    }

    @Override
    public FollowDetailVO followPoolDetail(FollowTask task) {
        FollowDetailVO vo = new FollowDetailVO();
        vo.setCustomerId(task.getCustomerId());
        vo.setType(task.getType().getValue());
        vo.setBizType(task.getBizType().getValue());
        vo.setTaskId(task.getId());
        vo.setDetailId(task.getDetailId());
        vo.setDeadline(task.getDeadline());
        vo.setTimes(task.getTimes());
        return vo;
    }


    /**
     * 待办终止时检查打电话情况标识数据
     *
     * @param record
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void markSecretReportHistory(FollowRecord record) {
        SecretReportHistory history = secretReportHistoryService.getOne(Wrappers.<SecretReportHistory>lambdaQuery()
                .eq(SecretReportHistory::getFollowRecordId, record.getId())
                .orderByDesc(SecretReportHistory::getCallDuration)
                .last(" limit 1")
        );

        if (Objects.nonNull(history)) {
            history.setReportMark(Boolean.TRUE);
            secretReportHistoryService.updateById(history);
        }
    }

    /**
     * 二次分配终止任务
     *
     * @param taskId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void secondTaskStopRecord(final Long taskId) {
        FollowRecord record = followRecordService.getOngoingRecord(taskId);
        if (Objects.isNull(record)) {
            return;
        }
        record.setReason(FollowCloseReason.LEAD_EXPIRES);
        record.setRemark(record.getRemark() + "+二次分配");
        record.setCloseTime(LocalDateTime.now());
        record.setFollowStatus(FollowStatusEnum.CLOSED);
        if (!Boolean.TRUE.equals(record.getAddTodo())) {
            record.setYn(Boolean.FALSE);
        }
        markSecretReportHistory(record);
        followRecordService.updateById(record);
        if (Boolean.TRUE.equals(record.getAddTodo())) {
            BackLogItemDTO dto = new BackLogItemDTO(record.getUserId(), record.getTodoCode(), record.getId().toString());
            todoRpcService.cancel(dto);
        }
    }

    /**
     * 重复校验
     *
     * @param originalData
     * @return
     */
    protected boolean duplicateCheck(OriginalData originalData) {
        int count = followTaskService.count(Wrappers.<FollowTask>lambdaQuery()
                .eq(FollowTask::getType, originalData.getType())
                .eq(FollowTask::getDetailId, originalData.getDetailId())
                .eq(FollowTask::getGroupId, originalData.getGroupId())
                .eq(FollowTask::getFollowUser, originalData.getUserId())
                .in(FollowTask::getState, TaskStateEnum.ONGOING)
        );
        return count > 0;
    }


    /**
     * 创建跟进待办基本信息
     * todocode、note、compeltel、refId需后续设置
     *
     * @param task
     * @return
     */
    protected FollowRecord createRecord(FollowTask task, Consumer<FollowRecord> cs) {
        FollowRecord record = new FollowRecord();
        record.setTaskId(task.getId());
        record.setBizType(task.getBizType());
        record.setType(task.getType());
        record.setCustomerId(task.getCustomerId());
        record.setCustomerName(task.getCustomerName());
        record.setContacts(task.getContacts());
        record.setMemberId(task.getMemberId());
        MemberUserDTO member = memberRpcService.user(task.getMemberId());
        if (Objects.nonNull(member)) {
            record.setContacts(member.getPhone());
        }
        record.setUserId(task.getFollowUser());
        UserInfoDTO user = ehrRpcService.user(task.getFollowUser());
        if (Objects.nonNull(user)) {
            record.setUserName(user.getUserName());
        }
        record.setFollowStatus(FollowStatusEnum.ONGOING);
        record.setGroupId(task.getGroupId());
        record.setShopId(task.getFollowShop());
        record.setAddTodo(Boolean.FALSE);
        record.setBizId(task.getBizId());
        record.setPlateNo(task.getPlateNo());
        record.setVin(task.getVin());
        record.setCalled(CallStateEnum.WAIT_DIAL);
        if (Objects.nonNull(cs)) {
            cs.accept(record);
        }
        return record;
    }

    @Deprecated
    protected FollowRecord createRecord(FollowTask task, LocalDateTime beginTime, LocalDateTime deadline) {
        FollowRecord record = new FollowRecord();
        record.setTaskId(task.getId());
        record.setBizType(task.getBizType());
        record.setType(task.getType());
        record.setCustomerId(task.getCustomerId());
        record.setCustomerName(task.getCustomerName());
        record.setMemberId(task.getMemberId());
        record.setUserId(task.getFollowUser());
        UserInfoDTO user = ehrRpcService.user(task.getFollowUser());
        if (Objects.nonNull(user)) {
            record.setUserName(user.getUserName());
        }
        record.setPlanTime(beginTime);
        record.setDeadline(deadline);
        record.setFollowStatus(FollowStatusEnum.ONGOING);
        record.setShopId(task.getFollowShop());
        record.setGroupId(task.getGroupId());
        record.setAddTodo(Boolean.FALSE);
        Map<String, String> map = todoProperty.getCodeMap();
        record.setTodoCode(map.get(record.getType().name()));
        record.setBizId(task.getBizId());
        record.setPlateNo(task.getPlateNo());
        assembleCompleteTelStatus(record, task);
        return record;
    }


    @Deprecated
    protected void updateContacts(FollowRecord record) {
        Long memberId = record.getMemberId();
        MemberUserDTO user = memberRpcService.user(memberId);
        if (Objects.nonNull(user)) {
            record.setContacts(user.getPhone());
        }
    }

    /**
     * 完成跟进时
     * 检查电话跟进状态
     *
     * @param record
     */
    protected void checkTelFollowStatus(FollowRecord record) {
        List<FollowRecordLog> list = followRecordLogService.list(Wrappers.<FollowRecordLog>lambdaQuery()
                .eq(FollowRecordLog::getRecordId, record.getId())
        );
        BV.isNotEmpty(list, () -> "无有效跟进记录");
        boolean match = list.stream().allMatch(log -> Objects.nonNull(log.getFeedbackType()));
        BV.isTrue(match, () -> "请检查跟进记录是否已完成客户反馈的录入");
        CallStateEnum called = record.getCalled();
        if (Boolean.TRUE.equals(record.getCompelTel())) {
            boolean effectiveDial = CallStateEnum.EFFECTIVE_DIAL.equals(called);
            BV.isTrue(effectiveDial, () -> "您必须完成不小于30秒的电话沟通，才能完成本次跟进！");
        } else {
            boolean ineffectiveDial = CallStateEnum.INEFFECTIVE_DIAL.equals(called);
            if (ineffectiveDial) {
                boolean haveAnyOtherFollow = list.stream().anyMatch(r -> !AttTypeEnum.SMART_PHONE.equals(r.getAttType()));
                BV.isTrue(haveAnyOtherFollow, () -> "由于电话沟通时长小于30秒，您必须补传至少一份其他凭证！");
            }
        }
    }

    /**
     * 检查是否需要覆盖并覆盖相关待办
     *
     * @param record
     * @return
     */
    protected boolean checkAndOverride(FollowRecord record) {
        record.setIgnoreReport(Boolean.FALSE);
        Long userId = record.getUserId();
        FollowTypeEnum type = record.getType();
        ArrayList<FollowTypeEnum> list = ListUtil.toList(FollowTypeEnum.AC, FollowTypeEnum.FA, FollowTypeEnum.SF);
        Set<FollowTypeEnum> typeSet = new HashSet<>(list);

        if (typeSet.contains(type) || (BizTypeEnum.PS.equals(record.getBizType()) && (Objects.isNull(record.getMemberId()) || record.getMemberId() <= 0))) {
            return followRecordService.save(record);
        }

        FollowRecord preOne = followRecordService.getOne(Wrappers.<FollowRecord>lambdaQuery()
                        .eq(FollowRecord::getUserId, userId)
                        .notIn(FollowRecord::getType, list)
                        .eq(BizTypeEnum.PS.equals(record.getBizType()), FollowRecord::getMemberId, record.getMemberId())
                        .eq(BizTypeEnum.AS.equals(record.getBizType()), FollowRecord::getVin, record.getVin())
                        .eq(FollowRecord::getBizType, record.getBizType())
                        .eq(FollowRecord::getFollowStatus, FollowStatusEnum.ONGOING)
                , false
        );

        if (Objects.isNull(preOne)) {
            return followRecordService.save(record);
        }
        if (preOne.getTaskId().equals(record.getTaskId())) {
            // 防止相同taskId重复创建跟进待办
            return false;
        }
        if (FollowTypeEnum.AF.equals(type)) {
            preOne.setFollowStatus(FollowStatusEnum.AC_OVERRIDE);
            preOne.setCloseTime(LocalDateTime.now());
            BackLogItemDTO dto = new BackLogItemDTO(preOne.getUserId(), preOne.getTodoCode(), preOne.getId().toString(), new Date(), preOne.getShopId());
            record.setRefId(preOne.getId());
            if (Boolean.TRUE.equals(preOne.getAddTodo())) {
                todoRpcService.cancel(dto);
            }
            preOne.setAddTodo(Boolean.TRUE);
            followRecordService.updateById(preOne);
            return followRecordService.save(record);
        }
        if (FollowTypeEnum.AF.equals(preOne.getType())) {
            record.setFollowStatus(FollowStatusEnum.AC_OVERRIDE);
            record.setAddTodo(Boolean.TRUE);
            record.setCloseTime(LocalDateTime.now());
            followRecordService.save(record);
            preOne.setRefId(record.getId());
            followRecordService.updateById(preOne);
            return true;
        }
        if (FollowTypeEnum.PL.equals(type)) {
            record.setFollowStatus(FollowStatusEnum.OT_OVERRIDE);
            record.setAddTodo(Boolean.TRUE);
            record.setCloseTime(LocalDateTime.now());
            followRecordService.save(record);
            preOne.setRefId(record.getId());
            followRecordService.updateById(preOne);
            return true;
        }
        preOne.setFollowStatus(FollowStatusEnum.OT_OVERRIDE);
        preOne.setCloseTime(LocalDateTime.now());
        BackLogItemDTO dto = new BackLogItemDTO(preOne.getUserId(), preOne.getTodoCode(), preOne.getId().toString(), new Date(), preOne.getShopId());
        record.setRefId(preOne.getId());
        if (Boolean.TRUE.equals(preOne.getAddTodo())) {
            todoRpcService.cancel(dto);
        }
        preOne.setAddTodo(Boolean.TRUE);
        followRecordService.updateById(preOne);
        return followRecordService.save(record);
    }

    protected <T extends FollowDetailVO> void assembleRecordDetail(T vo, FollowRecord record, FollowTask task) {
        vo.setId(record.getId());
        vo.setCustomerId(record.getCustomerId());
        vo.setCustomerName(record.getCustomerName());
        vo.setType(record.getType().getValue());
        vo.setBizType(record.getBizType().getValue());
        vo.setTimes(task.getTimes());
        vo.setTaskId(record.getTaskId());
        vo.setDeadline(record.getDeadline());
        vo.setUserId(record.getUserId());
        vo.setDetailId(task.getDetailId());
        vo.setMobile(record.getContacts());
        vo.setVin(task.getVin());
        vo.setRealMobile(record.getContacts());
        vo.setRegion(memberRpcService.attribution(record.getContacts()));
        //默认已经打过电话
        Boolean hadCall = Boolean.TRUE;
        if (Boolean.TRUE.equals(record.getCompelTel())) {
            long count = followRecordLogService.count(Wrappers.<FollowRecordLog>lambdaQuery()
                    .eq(FollowRecordLog::getTaskId, record.getTaskId())
                    .eq(FollowRecordLog::getAttType, AttTypeEnum.SMART_PHONE)
            );
            if (count <= 0) {
                hadCall = Boolean.FALSE;
            }
        }
        //是否已经打过电话
        vo.setHadCall(hadCall);
    }

    /**
     * 填充是否强制打电话的状态
     *
     * @param record
     * @param task
     */
    protected void assembleCompleteTelStatus(FollowRecord record, FollowTask task) {
        record.setCompelTel(Boolean.FALSE);
        if (task.getType().equals(FollowTypeEnum.CF) || task.getType().equals(FollowTypeEnum.PF) || task.getType().equals(FollowTypeEnum.AF)) {
            boolean noAF = task.getType().equals(FollowTypeEnum.CF) || task.getType().equals(FollowTypeEnum.PF);
            List<FollowRecord> list = followRecordService.list(Wrappers.<FollowRecord>lambdaQuery()
                    .eq(!noAF, FollowRecord::getMemberId, task.getMemberId())
                    .eq(noAF, FollowRecord::getCustomerId, task.getCustomerId())
                    .eq(FollowRecord::getUserId, task.getFollowUser())
                    .eq(FollowRecord::getFollowStatus, FollowStatusEnum.CLOSED)
                    .eq(FollowRecord::getAddTodo, Boolean.TRUE)
                    .eq(FollowRecord::getYn, Boolean.TRUE)
                    .in(FollowRecord::getType, FollowTypeEnum.AF, FollowTypeEnum.CF, FollowTypeEnum.PF)
                    .orderByDesc(FollowRecord::getTodoTime)
                    .last(" limit 2")
            );
            if (CollectionUtils.isEmpty(list)) {
                record.setCompelTel(Boolean.TRUE);
            } else if (list.size() == 2) {
                boolean anyMatchCalled = list.stream().anyMatch(r -> CallStateEnum.EFFECTIVE_DIAL.equals(r.getCalled()));
                record.setCompelTel(!anyMatchCalled);
            }
        }
    }

    /**
     * 更新反馈
     *
     * @param taskId
     * @param feedbackTypeEnum
     */
    protected void feedbackTask(Long taskId, FeedbackTypeEnum feedbackTypeEnum) {
        FollowTask task = followTaskService.getById(taskId);
        if (Objects.isNull(task) || Objects.isNull(feedbackTypeEnum)) {
            return;
        }
        String feedback = task.getFeedback();
        Set<String> feedbackSet = StringUtils.isEmpty(feedback) ? new HashSet<>() : new HashSet<>(Arrays.asList(feedback.split(",")));
        feedbackSet.add(String.valueOf(feedbackTypeEnum.getValue()));
        task.setFeedback(String.join(",", feedbackSet));
        task.setLastFeedback(feedbackTypeEnum);
        followTaskService.updateById(task);
    }


    /**
     * 原始数据跟进任务
     *
     * @param originalData
     * @return
     */
    private FollowTask transformTask(OriginalData originalData) {
        FollowTask task = new FollowTask();
        task.setType(originalData.getType());
        task.setBizType(originalData.getBizType());
        task.setMemberId(originalData.getMemberId());
        task.setContacts(originalData.getContacts());
        task.setCustomerId(originalData.getCustomerId());
        task.setCustomerName(originalData.getCustomerName());
        task.setVin(originalData.getFrameNo());
        task.setPlateNo(originalData.getPlateNo());
        task.setDetailId(originalData.getDetailId());
        task.setBizId(originalData.getBizId());
        task.setBeginTime(originalData.getGenerateTime());
        task.setFollowDuration(originalData.getFollowDuration());
        task.setDeadline(originalData.getDeadline());
        task.setState(TaskStateEnum.ONGOING);
        task.setFollowUser(originalData.getUserId());
        task.setFollowShop(originalData.getShopId());
        task.setTimes(0);
        task.setGroupId(originalData.getGroupId());
        task.setStopTodo(Boolean.FALSE);

        if (Objects.nonNull(originalData.getMemberId()) && originalData.getMemberId() > 0) {
            MemberUserDTO memberUserDTO = memberRpcService.user(originalData.getMemberId());
            if (Objects.nonNull(memberUserDTO)) {
                if (StringUtils.isValid(memberUserDTO.getRealName())) {
                    task.setCustomerName(memberUserDTO.getRealName());
                }
                task.setContacts(memberUserDTO.getPhone());
            }
        }

        return task;
    }
}
