package cn.fw.shirasawa.service.bus.follow.strategy;

import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.db.other.OriginalData;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.dto.FollowTaskCompleteDTO;
import cn.fw.shirasawa.domain.dto.FollowTerminationDTO;
import cn.fw.shirasawa.domain.dto.RecordCompleteDTO;
import cn.fw.shirasawa.domain.enums.FollowTypeEnum;
import cn.fw.shirasawa.domain.vo.follow.FollowDetailVO;
import cn.fw.shirasawa.domain.vo.follow.FollowRecordHistoryVO;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author : kurisu
 * @className : FollowStrategy
 * @description : 跟进策略接口
 * @date: 2020-08-14 10:10
 */
public interface FollowStrategy {
    /**
     * 获取跟进类型
     *
     * @return
     */
    FollowTypeEnum getFollowType();

    /**
     * 元数据生成任务
     *
     * @param originalData
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    boolean origin2task(OriginalData originalData);

    /**
     * 跟进待办逾期
     *
     * @param record
     */
    @Transactional(rollbackFor = Exception.class)
    void overdue(FollowRecord record);

    /**
     * 完成跟进并针对当前待办的跟进任务维护下次跟进时间
     *
     * @param record   跟进待办
     * @param userId   完成人
     * @param nextDate nextDate
     */
    @Transactional(rollbackFor = Exception.class)
    void complete(FollowRecord record, Long userId, LocalDate nextDate);

    /**
     * 完成跟进
     *
     * @param record  跟进待办
     * @param userId  完成人
     * @param nextDto 下次跟进时间
     */
    @Transactional(rollbackFor = Exception.class)
    void complete(FollowRecord record, Long userId, List<RecordCompleteDTO.NextDTO> nextDto);

    /**
     * 售前生成新的待办
     *
     * @param task
     * @param nextTime
     * @param nextEndTime
     * @param note
     * @deprecated ⚠️将在sdk版本1.2时废弃，后续不接受其他系统生成下次待办
     */
    @Transactional(rollbackFor = Exception.class)
    void createNewRecord(FollowTask task, LocalDateTime nextTime, LocalDateTime nextEndTime, String note);

    /**
     * 创建下次跟进待办
     *
     * @param task
     * @param nextTime
     */
    @Transactional(rollbackFor = Exception.class)
    void genNextRecord(FollowTask task, LocalDate nextTime);

    /**
     * 完成跟进任务
     *
     * @param taskCompleteDTO
     */
    @Transactional(rollbackFor = Exception.class)
    void completeTask(FollowTaskCompleteDTO taskCompleteDTO);

    /**
     * 终止跟进任务
     *
     * @param terminationDTO
     */
    @Transactional(rollbackFor = Exception.class)
    void terminationTaskPlan(FollowTerminationDTO terminationDTO);

    /**
     * 结束任务
     *
     * @param task
     * @param fromFlow 来自审批
     */
    @Transactional(rollbackFor = Exception.class)
    void endTask(FollowTask task, boolean fromFlow);

    /**
     * 查询跟进详情
     *
     * @param record
     * @return
     */
    FollowDetailVO queryDetail(FollowRecord record);

    /**
     * 查询跟进记录
     *
     * @param record
     * @return
     */
    List<FollowRecordHistoryVO> queryHistoryByRecord(FollowRecord record);

    /**
     * 查询跟进记录
     *
     * @param task
     * @return
     */
    List<FollowRecordHistoryVO> queryHistoryByTask(FollowTask task);

    /**
     * 上传跟进附件
     *
     * @param dto
     * @param record
     * @param userId
     */
    @Transactional(rollbackFor = Exception.class)
    void uploadAtt(FollowAttachmentDTO dto, FollowRecord record, Long userId);


    /**
     * 查询跟进池详情
     *
     * @param task
     * @return
     */
    FollowDetailVO followPoolDetail(FollowTask task);

    /**
     * 待办终止时检查打电话情况
     *
     * @param record
     */
    @Transactional(rollbackFor = Exception.class)
    void markSecretReportHistory(FollowRecord record);

    /**
     * 二次分配终止任务
     * @param taskId
     */
    @Transactional(rollbackFor = Exception.class)
    void secondTaskStopRecord(final Long taskId);
}
