package cn.fw.shirasawa.service.bus.follow.strategy.impl;

import cn.fw.common.util.ValidationUtils;
import cn.fw.shirasawa.common.utils.DateUtil;
import cn.fw.shirasawa.domain.db.follow.FollowRecord;
import cn.fw.shirasawa.domain.db.follow.FollowRecordLog;
import cn.fw.shirasawa.domain.db.follow.FollowTask;
import cn.fw.shirasawa.domain.dto.FollowAttachmentDTO;
import cn.fw.shirasawa.domain.dto.RecordCompleteDTO;
import cn.fw.shirasawa.domain.enums.*;
import cn.fw.shirasawa.domain.vo.follow.ACDetailVO;
import cn.fw.shirasawa.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.shirasawa.rpc.oop.dto.ShopDTO;
import cn.fw.shirasawa.rpc.valhalla.ValhallaRpcService;
import cn.fw.shirasawa.rpc.valhalla.dto.ValhallaCustomerDTO;
import cn.fw.shirasawa.service.bus.follow.strategy.AbstractFollowStrategy;
import cn.fw.shirasawa.service.event.RecordCompleteEvent;
import cn.fw.shirasawa.service.property.FollowTodoProperty;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : ACFollowStrategy
 * @description : 事故车策略
 * @date: 2020-08-17 10:48
 */
@Slf4j
@Component
public class ACFollowStrategy extends AbstractFollowStrategy {

    private final ValhallaRpcService valhallaRpcService;
    private final FollowTodoProperty followTodoProperty;

    @Autowired
    public ACFollowStrategy(final ValhallaRpcService valhallaRpcService,
                            final FollowTodoProperty followTodoProperty) {
        this.valhallaRpcService = valhallaRpcService;
        this.followTodoProperty = followTodoProperty;
    }

    @Override
    public FollowTypeEnum getFollowType() {
        return FollowTypeEnum.AC;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadAtt(FollowAttachmentDTO dto, FollowRecord record, Long userId) {
        if (Objects.isNull(record)) {
            return;
        }
        FollowTask task = followTaskService.getById(record.getTaskId());
        AttTypeEnum attTypeEnum = dto.getAttTypeEnum();
        FollowRecordLog recordLog = new FollowRecordLog();
        recordLog.setId(dto.getId());
        recordLog.setTaskId(record.getTaskId());
        recordLog.setAttachments(dto.getAttachments());
        recordLog.setAttType(attTypeEnum);
        recordLog.setFeedbackType(dto.getFeedbackTypeEnum());
        recordLog.setDescribes(dto.getDescribes());
        recordLog.setRecordId(dto.getRecordId());
        if (Objects.isNull(dto.getId())) {
            recordLog.setUploadTime(LocalDateTime.now());
        }
        if (AttTypeEnum.SMART_PHONE.equals(attTypeEnum)) {
            recordLog.setMark(Boolean.TRUE.equals(dto.getMark()));
        } else {
            recordLog.setMark(Boolean.TRUE);
        }
        followRecordLogService.save(recordLog);
        feedbackTask(record.getTaskId(), dto.getFeedbackTypeEnum());
        if (!dto.isAccComplete()) {
            return;
        }
        if (FollowStatusEnum.ONGOING.equals(record.getFollowStatus())) {
            record.setCloseTime(LocalDateTime.now());
            BackLogItemDTO backLogItemDTO = new BackLogItemDTO(record.getUserId(), record.getTodoCode(),
                    String.valueOf(record.getId()), DateUtil.toDate(LocalDateTime.now()), record.getShopId());

            record.setFollowStatus(FollowStatusEnum.CLOSED);
            record.setReason(FollowCloseReason.DONE);

            if (Objects.nonNull(task)) {
                int times = task.getTimes() == null ? 0 : task.getTimes();
                task.setTimes(times + 1);
                followTaskService.updateById(task);
            }
            todoRpcService.complete(backLogItemDTO);
            followRecordService.updateById(record);
            eventPublisher.publishEvent(new RecordCompleteEvent(record, task));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void secondTaskStopRecord(final Long taskId) {
        // 没有二次分配
    }

    @Override
    public ACDetailVO queryDetail(FollowRecord record) {
        FollowTask followTask = followTaskService.getById(record.getTaskId());
        BV.notNull(followTask, "跟进任务不存在");
        ACDetailVO vo = new ACDetailVO();
        assembleRecordDetail(vo, record, followTask);
        vo.setReportMobile(record.getContacts());
        vo.setPlateNo(record.getPlateNo());
        vo.setShopId(followTask.getFollowShop());
        ShopDTO shop = oopService.shop(followTask.getFollowShop());
        if (Objects.nonNull(shop)) {
            vo.setShopName(shop.getShortName());
        }
        boolean recordingAttachment = !ValidationUtils.checkMobile(vo.getReportMobile());
        vo.setRecordingAttachment(recordingAttachment);
        vo.setSecretMobile(recordingAttachment);
        vo.setTaskStartTime(followTask.getBeginTime());
        vo.setTaskDeadline(followTask.getDeadline());
        vo.setVin(followTask.getVin());
        ValhallaCustomerDTO dto = valhallaRpcService.queryByVin(record.getVin(), record.getGroupId());
        if (Objects.nonNull(dto)) {
            vo.setCustomerId(dto.getId());
        }
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void overdue(FollowRecord record) {
        super.overdue(record);
        long timeliness = Optional.ofNullable(followTodoProperty.getAccTodoTimeliness()).orElse(72).longValue();
        FollowRecord newRecord = createNextRecord(record, LocalDateTime.now(), timeliness);
        followRecordService.save(newRecord);
    }

    @Override
    public ACDetailVO followPoolDetail(FollowTask task) {
        ACDetailVO vo = new ACDetailVO();
        vo.setReportMobile(task.getContacts());
        vo.setPlateNo(task.getPlateNo());
        vo.setMobile(task.getContacts());
        vo.setRealMobile(task.getContacts());
        vo.setCustomerId(task.getCustomerId());
        vo.setCustomerName(task.getCustomerName());
        vo.setVin(task.getVin());
        vo.setType(task.getType().getValue());
        vo.setBizType(task.getBizType().getValue());
        vo.setTaskId(task.getId());
        vo.setDeadline(task.getDeadline());
        vo.setTimes(task.getTimes());
        boolean recordingAttachment = !ValidationUtils.checkMobile(vo.getReportMobile());
        vo.setRecordingAttachment(recordingAttachment);
        vo.setSecretMobile(recordingAttachment);
        ValhallaCustomerDTO dto = valhallaRpcService.queryByVin(task.getVin(), task.getGroupId());
        if (Objects.nonNull(dto)) {
            vo.setCustomerId(dto.getId());
        }
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void complete(FollowRecord record, Long userId, List<RecordCompleteDTO.NextDTO> nextDto) {
        super.complete(record, userId, nextDto);
        if (CollectionUtils.isEmpty(nextDto)) {
            return;
        }
        for (RecordCompleteDTO.NextDTO next : nextDto) {
            if (FollowTypeEnum.AC.equals(record.getType()) && record.getTaskId().equals(next.getTaskId())) {
                long timeliness = Optional.ofNullable(followTodoProperty.getAccTodoTimeliness()).orElse(72).longValue();
                LocalDateTime nextTime = Optional.ofNullable(next.getAccNextDate()).orElse(next.getNextDate().atTime(9, 0));
                FollowRecord newRecord = createNextRecord(record, nextTime, timeliness);
                newRecord.setDeadline(LocalDate.from(nextTime).atTime(23, 59, 59));
                newRecord.setCompelTel(Boolean.FALSE);
                followRecordService.save(newRecord);
            }
        }
    }

    private FollowRecord createNextRecord(FollowRecord record, LocalDateTime nextTime, long timeliness) {
        FollowRecord newRecord = new FollowRecord();
        BeanUtils.copyProperties(record, newRecord);
        newRecord.setId(null);
        newRecord.setIgnoreReport(Boolean.TRUE);
        newRecord.setRefId(null);
        newRecord.setPlanTime(nextTime);
        newRecord.setDeadline(record.getPlanTime().plusHours(timeliness));
        newRecord.setFollowStatus(FollowStatusEnum.ONGOING);
        newRecord.setCloseTime(null);
        newRecord.setReason(null);
        newRecord.setTodoTime(null);
        newRecord.setAddTodo(Boolean.FALSE);
        newRecord.setCalled(CallStateEnum.WAIT_DIAL);
        return newRecord;
    }
}
